package gov.nist.nlpir.irf.broker.braf;

import gov.nist.nlpir.irf.handle.Handle;
import gov.nist.nlpir.irf.handle.braf.DeDoubleBrafHandle;
import gov.nist.nlpir.misc.io.BufferedRandomAccessFile;
import gov.nist.nlpir.irf.de.DeDouble;

/** 
 * This class is a flat file persistent storage broker for DeDouble objects
 * <br><br>
 * 
 * @version $Revision: 1.1 $ 	
 * 
 * @author
 * This software was produced by NIST, an agency of the U.S. government,  
 * and by statute is not subject to copyright in the United States.        
 * Recipients of this software assume all responsibilities associated      
 * with its operation, modification and maintenance.  
 */

public class DeDoubleBrafBroker extends BufferedRandomAccessFileBroker {
  

  /** The static reference to the singleton broker for this class of objects*/
  private static DeDoubleBrafBroker broker = null;
  


  /** 
   * Returns the instance of the singleton class: doc file persistence 
   * broker, creating one first if one has not already been created.
   * @return the doc file persistence broker
   */      
  public static DeDoubleBrafBroker instance() {
    if (broker == null)
      broker = new DeDoubleBrafBroker();
    return broker;
  }
  
  

  /** 
   * Materializes the object associated with the designated argument.
   * This concrete version will handle any part of the materialization
   * which is specific to the object.
   * @param arg Information needed to materialize the object
   * @return Materialize the designated object
   */                 
  public Object currentRecordAsObject(Object arg) {
    DeDouble s = null;
    DeDoubleBrafHandle handle = (DeDoubleBrafHandle)arg;
    s = (DeDouble)readRecordFromFile(handle);
    return s;
  }
  


  /** 
   * Dematerializes the object associated with the designated argument.
   * This concrete version will handle any part of the materialization
   * which is specific to the object.
   * @param arg Object to be stored.
   * @return handle Information needed to materialize the object
   */                 
  public Handle currentObjectAsRecord(Object arg) {
    DeDoubleBrafHandle handle = new DeDoubleBrafHandle();
    writeRecordToFile(handle, arg);
    return handle;
  }
  


  /**
   * Overrides the default writeObject with tailored code
   * @parm obj object to write
   * @parm braf buffered random access file to write the object to
   */
  public void writeObject(Object obj, BufferedRandomAccessFile braf) throws 
  java.io.IOException {
    braf.writeDouble(((DeDouble)obj).getValue());
  }
  


  /**
   * Overrides the default readObject with tailored code
   * @parm braf buffered random access file to write the object to
   * @return the deserialized object
   */
  public Object readObject(BufferedRandomAccessFile braf) throws 
  java.io.IOException {
    return new DeDouble(braf.readDouble());
  }
}
