package gov.nist.nlpir.irf.broker.braf;

import gov.nist.nlpir.irf.handle.Handle;
import gov.nist.nlpir.irf.handle.braf.DeStringBrafHandle;
import gov.nist.nlpir.irf.de.DeString;
import gov.nist.nlpir.misc.io.BufferedRandomAccessFile;

/** 
 * This class is a flat file persistent storage broker for DeString objects
 * <br><br>
 *
 * @version $Revision: 1.1 $ 	
 *
 * @author
 * This software was produced by NIST, an agency of the U.S. government,  
 * and by statute is not subject to copyright in the United States.        
 * Recipients of this software assume all responsibilities associated      
 * with its operation, modification and maintenance.  
 */

public class DeStringBrafBroker extends BufferedRandomAccessFileBroker {
  

  /** The static reference to singleton broker for this class of objects */
  private static DeStringBrafBroker broker = null;
  


  /** 
   * Returns the instance of the singleton class: doc file persistence 
   * broker, creating one first if one has not already been created.
   * @return the doc file persistence broker
   */      
  public static DeStringBrafBroker instance() {
    if (broker == null)
      broker = new DeStringBrafBroker();
    return broker;
  }
  
 

  /** 
   * Materializes the object associated with the designated argument.
   * This concrete version will handle any part of the materialization
   * which is specific to the object.
   * @param arg Information needed to materialize the object
   * @return Materialize the designated object
   */                 
  public Object currentRecordAsObject(Object handle) {
    return readRecordFromFile((DeStringBrafHandle)handle);
  }
  


  /** 
   * Dematerializes the object associated with the designated argument.
   * This concrete version will handle any part of the materialization
   * which is specific to the object.
   * @param arg Object to be stored.
   * @return handle Information needed to materialize the object
   */                 
  public Handle currentObjectAsRecord(Object arg) {
    DeStringBrafHandle handle = new DeStringBrafHandle();
    writeRecordToFile(handle, arg);
    return handle;
  }
  

  /**
   * Overrides the default writeObject with tailored code
   * @parm obj object to write
   * @parm braf buffered random access file to write the object to
   */
  public void writeObject(Object obj, BufferedRandomAccessFile braf) 
    throws java.io.IOException {
    byte[] toBeStored = ((DeString)obj).getValue().getBytes();
    int lengthOfStorage = toBeStored.length;
    braf.writeInt(lengthOfStorage);
    braf.writeBytes(toBeStored, 0, lengthOfStorage);
  }
  

  /**
   * Overrides the default readObject with tailored code
   * @parm braf buffered random access file to write the object to
   * @return the deserialized object
   */
  public Object readObject(BufferedRandomAccessFile braf) throws 
  java.io.IOException {
    int lengthOfStorage = braf.readInt();
    byte[] readBytes = new byte[lengthOfStorage];
    braf.read(readBytes, 0, lengthOfStorage);
    return new DeString(new String(readBytes));
  }
}
