package gov.nist.nlpir.irf.broker.braf;

import java.io.IOException;

import gov.nist.nlpir.irf.handle.Handle;
import gov.nist.nlpir.irf.handle.braf.BufferedRandomAccessFileHandle;
import gov.nist.nlpir.irf.broker.PersistenceBroker;
import gov.nist.nlpir.irf.handle.braf.IdxFeatTextBrafHandle;
import gov.nist.nlpir.irf.proxy.PersistentObjectManager;

import gov.nist.nlpir.irf.feature.IdxFeatWordInText;
import gov.nist.nlpir.irf.document.ProxyDocument;
import gov.nist.nlpir.irf.de.ProxyDataElem;
import gov.nist.nlpir.irf.util.Support;
import gov.nist.nlpir.misc.io.BufferedRandomAccessFile;

/** 
 * This class represents a file-based broker for indexing features
 * <br><br>
 * 
 * @version $Revision: 1.1 $ 	
 * 
 * @author
 * This software was produced by NIST, an agency of the U.S. government,  
 * and by statute is not subject to copyright in the United States.        
 * Recipients of this software assume all responsibilities associated      
 * with its operation, modification and maintenance.  
 *                             
 */

public class IdxFeatTextBrafBroker extends IdxFeatStringBrafBroker {

  /** The static reference to singleton broker for this class of objects */
  private static IdxFeatTextBrafBroker broker = null;
  


  /**
   * Returns the instance of the singleton class; IF file persistence
   * broker, creating the a broker instance if it doesn't already
   * exist.
   * @returns instance of IF file persistence broker.
   */
  public static PersistenceBroker instance() {
    if (broker == null)
      broker = new IdxFeatTextBrafBroker();
    return broker;
  }



  /** 
   * Materializes the object associated with the designated argument.
   * This concrete version will handle any part of the materialization
   * which is specific to the object.
   * @param arg Information needed to materialize the object
   * @return Materialize the designated object
   */                 
  public Object currentRecordAsObject(Object handle) {
    return readRecordFromFile((IdxFeatTextBrafHandle)handle);
  }
  


  /** 
   * Dematerializes the object associated with the designated argument.
   * This concrete version will handle any part of the materialization
   * which is specific to the object.
   * @param arg Object to be stored.
   * @return handle Information needed to materialize the object
   */                 
  public Handle currentObjectAsRecord(Object arg) {
    IdxFeatTextBrafHandle handle = new IdxFeatTextBrafHandle();
    writeRecordToFile(handle, arg);
    return handle;
  }
  


  /** 
   * Dematerializes the object associated with the designated argument
   * in a compact form.
   * @param arg Object to be stored.
   */   
  public void writeObject(Object obj, BufferedRandomAccessFile braf) 
    throws IOException {
    super.writeObject(obj, braf);
    braf.writeInt(((IdxFeatWordInText)obj).getParagraphCount());
  }



  /** 
   * Materializes the object from the specified file. 
   * See writeObject() for details on the convention used to (re)store
   * the object.
   * @param braf a buffered random access file
   * @return object materialized object that this broker is responsible for
   */     
  public Object readObject(BufferedRandomAccessFile braf) throws IOException {
    ProxyDataElem feature = PersistentObjectManager.buildDE(braf);
    ProxyDocument sourceDoc = PersistentObjectManager.buildDoc(braf);
    ProxyDataElem sourceField = PersistentObjectManager.buildDE(braf);

    int wordCount = braf.readInt();
    int paragraphCount = braf.readInt();
    
    IdxFeatWordInText result = new IdxFeatWordInText(feature, sourceDoc, sourceField,
					     wordCount, paragraphCount);
    result.setProxyRefsCountedTrue();
    return result;
  }
}
