package gov.nist.nlpir.irf.broker.braf;

import gov.nist.nlpir.irf.handle.Handle;
import gov.nist.nlpir.irf.handle.braf.IndexBrafHandle;
import gov.nist.nlpir.irf.broker.braf.BufferedRandomAccessFileBroker;
import gov.nist.nlpir.irf.handle.braf.BufferedRandomAccessFileHandle;
import gov.nist.nlpir.misc.io.BufferedRandomAccessFile;

/** 
 * This class is a file-based broker for Index objects
 * <br><br>
 * 
 * @version $Revision: 1.1 $ 	
 * 
 * @author
 * This software was produced by NIST, an agency of the U.S. government,  
 * and by statute is not subject to copyright in the United States.        
 * Recipients of this software assume all responsibilities associated      
 * with its operation, modification and maintenance.  
 */

public class IndexBrafBroker extends BufferedRandomAccessFileBroker {
  
  /** The static reference to singleton broker for this class of objects */
  private static IndexBrafBroker broker = null;
  


  /** 
   * Returns the instance of the singleton class: index file persistence 
   * broker, creating one first if one has not already been created.
   * @return the broker
   */      
  public static IndexBrafBroker instance() {
    if (broker == null)
      broker = new IndexBrafBroker();
    return broker;
  }
  

  
  /** 
   * Materializes the object associated with the designated argument.
   * This concrete version will handle any part of the materialization
   * which is specific to the object.
   * @param arg Information needed to materialize the object
   * @return Materialize the designated object
   */                 
  public Object currentRecordAsObject(Object arg) {
    return readRecordFromFile((IndexBrafHandle)arg);
  }
  


  /** 
   * Dematerializes the object associated with the designated argument.
   * This concrete version will handle any part of the materialization
   * which is specific to the object.
   * @param arg Object to be stored.
   * @return handle Information needed to materialize the object
   */                 
  public Handle currentObjectAsRecord(Object arg) {
    IndexBrafHandle handle = new IndexBrafHandle();
    writeRecordToFile(handle, arg);
    return handle;
  }



  /** 
   * Dematerializes the updated object associated with the designated 
   * argument.
   * This concrete version will handle any part of the materialization
   * which is specific to the object.
   * @param oldHandle Handle of the object prior to update
   * @param arg Object to be stored.
   * @return handle Information needed to materialize the updated object
   */                 
  public Handle currentObjectAsRecord(Handle oldHandle, Object arg) {
    // Someday the old handle should be used to mark the earlier copy
    // as no longer needed.
    IndexBrafHandle handle = new IndexBrafHandle();
    writeRecordToFile(handle, arg);
    return handle;
  }
}
