package gov.nist.nlpir.irf.conversion;

import java.io.Reader;
import java.io.IOException;
import java.io.FileNotFoundException;

import gov.nist.nlpir.irf.document.ProxyDocument;

/**
 * This abstract class models generic features of converters, which take
 * "raw" data and turn it into Document objects.
 *
 * @author
 * This software was produced by NIST, an agency of the U.S. government,  
 * and by statute is not subject to copyright in the United States.        
 * Recipients of this software assume all responsibilities associated      
 * with its operation, modification and maintenance.
 * @version $Revision: 1.1 $
 */

public abstract class IrfConverter {


  /** An example of the class of proxy IR documents to be created */
  private Class proxyDocClass;


  /** An example of the class of real IR documents to be created */
  private Class realDocClass;



  /**
   * Sets the value of the real document class given a class name
   * @parm the real document class name
   */
  public void setRealDocClass(String realDocClassName) {
    try {
      this.realDocClass = Class.forName(realDocClassName);
    } catch(ClassNotFoundException cnfe) {
      System.err.println("Error during Bib2AppDocConv(): " + cnfe);
    }
  }
  


  /** 
   * Returns the value of the real document class
   * @return the value of the real document class 
   */
  public Class getRealDocClass() {
    return realDocClass;
  }



  /**
   * Sets the value of the proxy document class given a class name
   * @parm the proxy document class name
   */
  public void setProxyDocClass(String proxyDocClassName) {
    try {
      this.proxyDocClass = Class.forName(proxyDocClassName);
    } catch(ClassNotFoundException cnfe) {
      System.err.println("Error during Bib2AppDocConv(): " + cnfe);
    }
  }



  /** 
   * Returns the value of the proxy document class
   * @return the value of the proxy document class 
   */
  public Class getProxyDocClass() {
    return proxyDocClass;
  }



  /**
   * Gives the converter the information it needs to access the raw
   * data for the documents to be converted.
    * @param loc string with info on where raw documents can be found
   */
  public abstract void setRawDocLocation(String loc) throws 
  FileNotFoundException;



  /**
   * Converts one raw document's worth of data into its object counterpart
   * @return a proxy ir document with its associated real ir document
   */
  public abstract ProxyDocument convert1()
    throws IOException;



  /**
   * Starting from the current location in the input stream, skips the
   * specified number of documents
   * @param numToSkip number of documents to skip
   * @return the number of documents actually skipped
   */
  public abstract int ignoreN(int numToSkip)
    throws IOException;
}
