package gov.nist.nlpir.irf.de;

/**
 * Internal word counter for DeHtml.
 *
 * @author 
 * This software was produced by NIST, an agency of the U.S. government,
 * and by statute is not subject to copyright in the United States.
 * Recipients of this software assume all responsibilities associated
 * with its operation, modification and maintenance.
 *
 * @version $Revision: 1.1 $
 * @see DeHtml
 */

public class HtmlAddrCount extends Object {

  /** Number of the current word in the current paragraph, starting at 0
      but usually newWord() is called before the word number is used, so
      that the actual start can be considered 1. */
  int wordCount;


  /** Number of the current paragraph, starting at 1. */
  int paragraphCount;
  

  /**
   * Initializes counters with new HtmlAddrCount Object.
   */
  public HtmlAddrCount() {
    wordCount = 0;
    paragraphCount = 1;
  }
  

  /**
   * Increments paragraph counter and reset word counter if paragraph
   * tag is supplied.
   * @param sgmltag a sgml tag.
   */
  public final void newTag(String sgmltag) {
    String tag = sgmltag.substring(0, sgmltag.length() - 1);
    if (tag.equalsIgnoreCase("p")) {
      paragraphCount++;
      wordCount = 0;
    }
  }  



  /**
   * Increments word counter.
   */
  public final void newWord() {
    wordCount++; 
  }



  /**
   * Returns the number of the current word within the current paragraph
   * @return the number of the current word within the current paragraph
   */
  public final int getWordCount() {
    return wordCount;
  }
 


  /**
   * Returns the number of the current paragraph
   * @return the number of the current paragraph
   */
  public final int getParagraphCount() {
    return paragraphCount;
  }



  /**
   * Writes the current word and paragraph counters' values to standard out
   */
  public void present() {
    System.out.println("HtmlAddrCount: word count = " + wordCount +
		       "\nHtmlAddrCount: paragraph count = " + paragraphCount); 
  }
}
