package gov.nist.nlpir.irf.de;

import gov.nist.nlpir.irf.index.braf.DeIntern;
import gov.nist.nlpir.irf.util.Support;
import gov.nist.nlpir.irf.proxy.VirtualProxy;
import gov.nist.nlpir.irf.proxy.Oid;

/** 
 * This is a class which represents what's common to all proxies
 * for data elements - not much. 
 * <br><br>
 * 
 * @version $Revision: 1.2 $ 	
 * 
 * @author
 * This software was produced by NIST, an agency of the U.S. government,  
 * and by statute is not subject to copyright in the United States.        
 * Recipients of this software assume all responsibilities associated      
 * with its operation, modification and maintenance.
 */

public abstract class ProxyDataElem extends VirtualProxy implements DataElem {

  /**
   * Tests whether the real data element is identical in its value to the
   * the one passed.
   * @return boolean the result of the test
   */
  public boolean equals(Object o) {
    /* Optimization using the Oid instead of materializing the real 
     * object each time: More often, it will be called on a DE that does 
     * have an Oid than with a DE that may not have one. The code reflects
     * this and should help during indexing and update. */
    if (o instanceof ProxyDataElem) {
      Oid otherID = ((ProxyDataElem)o).getObjectIdentifier();
      if (otherID != null) { // betting this will more often true than not
	if (objectIdentifier != null)
	  return objectIdentifier.equals(otherID);
      } else
	return ((DataElem)getRealObject()).
	  equals(((ProxyDataElem)o).getRealObject());
    } else if (o instanceof DataElem)
      return getRealObject().equals((DataElem)o);
    else if (o instanceof DeIntern)
      return getRealObject().equals(((DeIntern)o).getDE());
    
    return false;
  }
}


