package gov.nist.nlpir.irf.de;

import java.util.Vector;

import gov.nist.nlpir.irf.de.DataElem;
import gov.nist.nlpir.irf.document.ProxyDocument;
import gov.nist.nlpir.irf.modality.IndexingModalities;

import gov.nist.nlpir.irf.broker.PersistenceBroker;
import gov.nist.nlpir.irf.broker.braf.DeDoubleBrafBroker;

/**
 *  This class represents a DeDouble by proxy.
 *
 *  @version $Revision: 1.2 $ 	
 *  @author
 *  This software was produced by NIST, an agency of the U.S. government,  
 *  and by statute is not subject to copyright in the United States.        
 *  Recipients of this software assume all responsibilities associated      
 *  with its operation, modification and maintenance.  
 */
public class ProxyDeDouble extends ProxyDataElem {
    
  /** Constructs an instance from no arguments */
  public ProxyDeDouble() {
  }
  


  /**
   * Builds a ProxyDeDouble that represents the given a DeDouble.
   * @param v the value that the DeDouble will represent.
   */
  public ProxyDeDouble(DeDouble v) {
    setRealObject(v);
  }


  
  /**
   * Builds a ProxyDeDouble that represents the given value.
   * @param v the value that the DeDouble will represent.
   */
  public ProxyDeDouble(double v) {
    setRealObject(new DeDouble(v));
  }
  
  

  /**
   * Builds a ProxyDeDouble that represents the given a string representing
   * a double
   * @param v the value that the DeDouble will represent, represented as
   * a string.
   */
  public ProxyDeDouble(String v) {
    setRealObject(new DeDouble(v));
  }
  


  /**
   * Returns the value of the real DeDouble
   * @return the value of the real DeDouble
   */
  public double getValue() {
    return ((DeDouble)getRealObject()).getValue();
  }



  /**
   * Sets the value of the real DeDouble
   * @param v the new value the DeDouble represents.
   */
  public final void setValue(double v) { 
    ((DeDouble)getRealObject()).setValue(v);
  }
    


  /**
   * Displays the real DeDouble (surrounded with spaces)
   */
  public final void present() { 
    ((DeDouble)getRealObject()).present();
  }
    


  /**
   * Same as present(), but with no space.
   */
  public void presentBrief() { 
    ((DeDouble)getRealObject()).presentBrief();
  }
    


  /**
   * Scores the match of the real DeDouble with another. For the function
   * to succeed, the object must be a DeDouble also.
   * @param de the DeDouble this is compared with.
   * @return the score of the matching process.
   */
  public float match(DataElem de) { 
    return ((DeDouble)getRealObject()).match(de);
  }



  /**
   * Compares the real DeDouble to another. Definition would be 
   * ExactMatch(DeDouble) if java allowed it.
   * @param de the DeDouble this is compared with.
   * @return true if the values are similar, <br>
   * false otherwise or if de is not a DeDouble.
   */
  public boolean exactMatch(DataElem de) { 
    return ((DeDouble)getRealObject()).exactMatch(de);
  }



  /**
   * Extract indexing features from the real DeDouble
   * @param s source document of data element.
   * @param im the controlling indexing modalities
   * @param field the document field from which the real data element came
   * @return Vector of indexing features for this.
   */
  public Vector getIndexingFeatures(ProxyDocument source, 
				    IndexingModalities im,
				    DataElem field) {
    return ((DeDouble)getRealObject()).getIndexingFeatures(source, 
							    im, 
							    field);
  }



  /**
   * Returns a string representation of the real DeDouble
   * @return string representation of the real DeDouble
   */
  public String getValueAsString() {
    return ((DeDouble)getRealObject()).getValueAsString();
  }



  /**
   * Returns a string representation of the real DeDouble
   * @return string representation of the real DeDouble
   */
  public String toString() {
    return ((DeDouble)getRealObject()).toString();
  }



  /**
   * Scores the comparison between the real DeDouble and the given one.
   * @return the score as an int. It's the integer part of the difference
   * between the two values. The result is Integer.MAX_VALUE if o2 is not
   * a DeDouble.
   * @param o the DeDouble to compare to.
   */
  public int compareTo(DataElem o) {
    return ((DeDouble)getRealObject()).compareTo(o);
  }



  /**
   * Redefines the default object.hashCode(). The resulting hash code 
   * is based on the one of the represented double value.
   *  @return hash code of DeDouble, used for Index, and Hashtable
   * @see java.util.Hashtable
   * @see java.lang.Object
   */
  public int hashCode() { 
    return ((DeDouble)getRealObject()).hashCode();
  }
    


  /**
   * Adds the value of the real DeDouble to the HTML text given.
   * @param string the HTML text the value will be appended to.
   * @param i unused.
   * @param form unused.
   */
  public void addHTML(StringBuffer baseString, int i, boolean form) { 
    ((DeDouble)getRealObject()).addHTML(baseString, i, form);
  }



  /**
   * Appends the value to the given string in order to create an HTML doc.
   * @param baseString the string which the value is added to.
   */
  public void HTML_ShowDoc(StringBuffer baseString) {
    ((DeDouble)getRealObject()).HTML_ShowDoc(baseString);
  }



  /** 
   * Creates a broker for the real DeDouble and remember it
   * @return the broker for this object
   */          
  public PersistenceBroker createBroker() {
    setBroker(DeDoubleBrafBroker.instance());
    return getBroker();
  }
}
