package gov.nist.nlpir.irf.de;

import gov.nist.nlpir.irf.de.DeHtml;
import gov.nist.nlpir.irf.modality.IndexingModalities;
import gov.nist.nlpir.irf.document.ProxyDocument;
import java.util.Vector;
import gov.nist.nlpir.irf.broker.PersistenceBroker;
import gov.nist.nlpir.irf.broker.braf.DeHtmlBrafBroker;


/**
 * This class represents a DeHtml by proxy.<br><br>
 *
 * @author 
 * This software was produced by NIST, an agency of the
 * U.S. government, and by statute is not subject to copyright in the
 * United States.  Recipients of this software assume all
 * responsibilities associated with its operation, modification and
 * maintenance.
 * @version $Revision: 1.2 $
 */
public class ProxyDeHtml extends ProxyDeString {
 

  /** Constructs an instance from no arguments */
  public ProxyDeHtml() {
  }



  /**
   * Creates a ProxyDeHtml object, given a DeHtml object.
   * @param v the string, which may content HTML tags.
   */
  public ProxyDeHtml(DeHtml  v) {
    setRealObject(v);
  }
  


  /**
   * Creates a ProxyDeHtml object, given a String object
   * @param v the string, which may content HTML tags.
   */
  public ProxyDeHtml(String v) {
    setRealObject(new DeHtml(v));
  }
  


  /**
   * Creates a ProxyDeHtml object, given an array of chars
   * @param v the array of char corresponding to an HTML string.
   */
  public ProxyDeHtml(char v[]) {
    setRealObject(new DeHtml(v));
  }
  


  /**
   * Computes the score given by the comparison between teh real DeHtml
   * and the given DeHtml
   * @return the score as an int. It's the difference between the HTML 
   * string values
   * @param o the DeHtml this is compared to
   */
  public int compareTo(DataElem o) {
    return ((DeHtml)getRealObject()).compareTo(o);
  }
  


  /**
   * Extracts Indexing Features from the real DeHtml.
   * @param source object reference of source document.
   * @param im indexing modalities to be used with this DeString.
   * @param field the document field from which this data element came
   * @return vector of IndexingFeatures.
   */
  public Vector getIndexingFeatures(ProxyDocument source, 
				    IndexingModalities im,
				    DataElem field) {
    return ((DeHtml)getRealObject()).getIndexingFeatures(source, im, field);
  }
  

  
  /**
   *  Adds the real current DeHtml value to the HTML text given  
   *  @param baseString the HTML text the value will be appended to
   *  @param maxNrOfChar maximum number of characters appended. 
   *  If 0, no limit to this number
   *  @param form if true, the string will be used to create a form, 
   *  a common text otherwise
   *  @return the new HTML string
   */
  public void addHTML(StringBuffer baseString, int maxNrOfChar,boolean form){
    ((DeHtml)getRealObject()).addHTML(baseString, maxNrOfChar, form);
  }
  


  /**
   * Appends the real DeHtml to the given string.
   * @param baseString the HTML text this  will be added to.
   */
  public void HTML_ShowDoc(StringBuffer baseString) {
    ((DeHtml)getRealObject()).HTML_ShowDoc(baseString);
  }
  


  /** 
   * Creates a broker for this proxy object and remember it
   * @return the broker for this object
   */          
  public PersistenceBroker createBroker(){
    setBroker(DeHtmlBrafBroker.instance());
    return getBroker();
  }
}
