package gov.nist.nlpir.irf.de;

import java.util.Vector;
import gov.nist.nlpir.irf.de.DeInteger;
import gov.nist.nlpir.irf.document.ProxyDocument;
import gov.nist.nlpir.irf.modality.IndexingModalities;
import gov.nist.nlpir.irf.broker.PersistenceBroker;
import gov.nist.nlpir.irf.broker.braf.DeIntegerBrafBroker;


/**
 * Proxy representation of a integer as a DataElem. <br><br>
 *  
 * @version $Revision: 1.2 $ 	
 * @author
 * This software was produced by NIST, an agency of the U.S. government,  
 * and by statute is not subject to copyright in the United States.        
 * Recipients of this software assume all responsibilities associated      
 * with its operation, modification and maintenance.  
 */                      

public class ProxyDeInteger extends ProxyDataElem {
  
  /**
   * No-argument constructor
   */
  public ProxyDeInteger() {
  }
  


  /**
   * Creates a ProxyDeInteger that represents the given integer.
   * @param v the value of the represented integer as DeInteger.
   */
  public ProxyDeInteger(DeInteger v) {
    setRealObject(v);
  }
  


  /**
   * Creates a ProxyDeInteger that represents the given integer.
   * @param v the value of the represented integer.
   */
  public ProxyDeInteger(int v) {
    setRealObject(new DeInteger(v));
  }
  


  /**
   * Creates a ProxyDeInteger that represents the given integer.
   * @param v the value of the  integer as a string.
   */
  public ProxyDeInteger(String v) {
    setRealObject(new DeInteger(v));
  }
  


  /**
   * Return the value of the represented integer
   * @return the value of the represented integer.
   */
  public final int getValue() { 
    return ((DeInteger)getRealObject()).getValue();
  }
  


  /**
   * Sets the value of the represented integer
   * @param v the new value of the represented integer.
   */
  public final void setValue(int v) {
    ((DeInteger)getRealObject()).setValue(v); 
  }
  


  /**
   * Prints the value of the represented integer.
   */
  public void present() {
    ((DeInteger)getRealObject()).present();
  }
  


  /**
   * Exactly the same thing as present(): prints the value.
   */
  public void presentBrief() {
    ((DeInteger)getRealObject()).presentBrief();
  }
  


  /**
   * Returns a string representation of the integer
   * @return string representation of the integer. Same as getAsString().
   */
  public final String getValueAsString() {
    return ((DeInteger)getRealObject()).getValueAsString();
  }
  


  /**
   * Returns a string representation of the integer
   * @return string representation of the integer.
   */
  public final String toString() {
    return ((DeInteger)getRealObject()).toString();
  }
  


  /**
   * Gets the (actually unique) indexing feature of this ProxyDeInteger.
   * @param source the Document in which the integer was read.
   * @param im the controlling indexing modalities
   * @param the document field from which this data element came
   * @return a vector with one element: an indexing feature 
   */
  public Vector getIndexingFeatures(ProxyDocument source,
				    IndexingModalities im,
				    DataElem field){
    return ((DeInteger)getRealObject()).getIndexingFeatures(source, 
							     im, 
							     field);
  }
    
  /**
   * Compares the values of the real DE_Intger and the given DeInteger 
   * and sets the result.
   * @param de the DeInteger to compare to
   * @return true if the real DE_Intger and de have the same value,<br>
   * false otherwise or if de is not a DeInteger.
   */
  public boolean exactMatch(DataElem de) {
    return ((DeInteger)getRealObject()).exactMatch(de);
  }
  


  /**
   * Computes the matching score between the real DeInteger and the given 
   * DeInteger. 
   * Actually, it's 1 if both DeIntegers have the same value, 0 otherwise.
   * @param de the DeInteger this is compared to.
   * @return 1 if this and de have the same value,<br>
   * 0 otherwise.
   */
  public float match(DataElem de) {
    return ((DeInteger)getRealObject()).match(de);
  }
  


  /**
   * Computes the difference between the real DeInteger and the given 
   * DeInteger.
   * The result is the mathematically defined difference.
   * @param o the DeInteger this is compared to.
   * @return the difference between this and compareTo, Integer.MAX_VALUE
   * if compareTo is not a DeInteger.
   */
  public int compareTo(DataElem o) {
    return ((DeInteger)getRealObject()).compareTo(o);
  }
  


  /**
   * Redefines the default Object.hashCode().
   * @return hash code of DeInteger, used for Index, and Hashtable.
   * @see java.util.Hashtable
   * @see java.lang.Object
   */
  public int hashCode() {
    return ((DeInteger)getRealObject()).hashCode();
  }
  


  /**
   * Appends the string representation of this to the given HTML text.<br>
   * Caution: string is modified by this function.
   * @param baseString the text to be extended by this. Caution: string 
   * itself is modified.
   * @param i not used.
   * @param form not used.
   * @return the new string.
   */
  public void addHTML(StringBuffer baseString, int i, boolean form) {
    ((DeInteger)getRealObject()).addHTML(baseString, i, form);
  }
  


  /**
   * Appends a string representation of the real DeInteger to the 
   * given string.
   * @param baseString the string to be completed.
   */
  
  public void HTML_ShowDoc(StringBuffer baseString) {
    ((DeInteger)getRealObject()).HTML_ShowDoc(baseString);
  }
  


  /** 
   * Creates a  broker for this proxy object and remembers it
   * @return the broker for this object
   */          
  public PersistenceBroker createBroker(){
    setBroker(DeIntegerBrafBroker.instance());
    return getBroker();
  }
}
