package gov.nist.nlpir.irf.de;

import java.util.Vector;

import gov.nist.nlpir.irf.modality.IndexingModalities;
import gov.nist.nlpir.irf.document.ProxyDocument;

import gov.nist.nlpir.irf.broker.PersistenceBroker;
import gov.nist.nlpir.irf.broker.braf.DePersonNameBrafBroker;

/**
 * Instances of this class represent a DePersonName by proxy
 *  
 *  @version $Revision: 1.2 $ 	
 *  @author
 *  This software was produced by NIST, an agency of the U.S. government,  
 *  and by statute is not subject to copyright in the United States.        
 *  Recipients of this software assume all responsibilities associated      
 *  with its operation, modification and maintenance.  
 */

public class ProxyDePersonName extends ProxyDataElem {

  /** No-argument constructor */
  public ProxyDePersonName() {
  }
  


  /**
   * Builds a ProxyDePersonName given a DePersonName assumed to 
   * copntain the surname. The first name will be null.
   * @param sur the surname of the person.
   */
  public ProxyDePersonName(DePersonName sur) {
    setRealObject(sur);
  }
  


  /**
   * Builds a DePersonName assuming the only string given is the person's
   * surname. The first name will be null.
   * @param sur the surname of the person.
   */
  public ProxyDePersonName(String sur) {
    setRealObject(new DePersonName(sur));
  }
  


  /**
   * Builds a DePersonName considering the only array of characters given as
   * a String representing the person's surname. The first name will be null.
   * @param sur considered as a string, will be the last name of the person.
   */
  public ProxyDePersonName(char sur[]) {
    setRealObject(new DePersonName(sur));
  }
  


  /**
   *  Assigns first and last name.
   *  @param first person's first name.
   *  @param sur person's surname.
   */
  public ProxyDePersonName(String first, String sur) {
    setRealObject(new DePersonName(first,sur));
  }
  


  /**
   * Treats the two arrays of characters as Strings and creates the 
   * DePersonName associated.
   */
  public ProxyDePersonName(char first[], char sur[]) {
    setRealObject(new DePersonName(first,sur));
  }
  


  /**
   * Retrieves the first name as a string.
   */
  public final String getFirstName() {
    return ((DePersonName)getRealObject()).getFirstName();
  }
  


  /**
   * Retrieves the surname as a string.
   */
  public final String getSurname() {
    return ((DePersonName)getRealObject()).getSurname();
  }
  


  /**
   * Sets the two main variables of the DePersonName.
   */
  public final void setValue(String first, String sur) {
    ((DePersonName)getRealObject()).setValue(first,sur);
  }
  


  /**
   * Prints a presentation of the DePersonName like this:<pre>
   * Name: myname
   * FirstName: myfirstName</pre>
   */
  public void present() {
    ((DePersonName)getRealObject()).present();
  }


    
  /**
   * Here, the presentation is briefer than with present():<pre>
   * myfirstName myname, </pre>
   */
  public void presentBrief() {
    ((DePersonName)getRealObject()).presentBrief();
  }
    


  /**
   * Compares this DePersonName to another.
   * @param de the DePersonName this is compared with.
   * @return true if both names and first names are the same,<br>
   * false otherwise or if de is not a DePersonName.
   */
  public boolean exactMatch(DataElem de) {
    return ((DePersonName)getRealObject()).exactMatch(de);
  }


  
  /**
   * Scores the match of this DePersonName with another Object. 
   * For the function to succeed, the object must be a DePersonName also. 
   * Actually the result is 1 if the two objects exactly match, 0 otherwise.
   * @param de the DePersonName this is compared with.
   * @return the score of the matching process.
   */
  public float match(DataElem de) {
    return ((DePersonName)getRealObject()).match(de);
  }
  


  /**
   * Overrides of Object.hashCode(), based on the hashCoding of the surname.
   * @return hash code of DeDouble, used for Index, and Hashtable.
   * @see java.util.Hashtable
   * @see java.lang.Object
   */
  public int hashCode() {
    return ((DePersonName)getRealObject()).hashCode();
  }
  


  /**
   * Scores the comparison between this and the given DePersonName
   * @return the score as an int:<br>
   * If surnames are not the same, returns the comparison between them.<br>
   * If they are, returns the comparison between first names.<br>
   * In every case, if o is not a DePersonName, returns 0.
   * @param o the DePersonName this is compared to.
   * @see String#compareTo
   */
  public int compareTo(DataElem o) {
    return ((DePersonName)getRealObject()).compareTo(o);
  }


  
  /**
   * Extract indexing features from this data element
   * @param source source document of data element
   * @param im the controling indexing modalities
   * @param field the document field from which this data element came
   * @return collection of indexing features for data element.
   */
  public Vector getIndexingFeatures(ProxyDocument source, 
				    IndexingModalities im,
				    DataElem field) {
    return ((DePersonName)getRealObject()).getIndexingFeatures(source, im,
								field);
  }


  
  /**
   * Gives the same result as toString().
   * @return string representation of value of object.
   */
  public final String getValueAsString() {
    return ((DePersonName)getRealObject()).getValueAsString();
  }
    


  /**
   * Returns a string representation of the DePersonName
   * @return string representation of object.
   * <pre>surname, firstname</pre>
   */
  public final String toString() {
    return ((DePersonName)getRealObject()).toString();
  }
    


  /**
   * Adds the name and surname to the HTML text given. If it is to be read
   * in a form, the HTML code will allow the entry of values.
   * @param baseString the HTML text the value will be appended to.
   * @param maxNrOfChar maximum number of characters added, used only 
   * if form is false.
   * @param form true if the HTML code must be used in a form.
   * @return the new HTML string.
   */
  public void addHTML(StringBuffer baseString, int maxNrOfChar, boolean form) {
    ((DePersonName)getRealObject()).addHTML(baseString, maxNrOfChar, form);
  }
    


  /**
   * Adds the name and surname of the person to the HTML string given
   * (in this order).
   * @param baseString the String the two names will be added.
   */
  
  public void HTML_ShowDoc(StringBuffer baseString) {
    ((DePersonName)getRealObject()).HTML_ShowDoc(baseString);
  }



  /** 
   * Creates a broker for this proxy object and remember it
   * @return the broker for this object
   */          
  public PersistenceBroker createBroker(){
    setBroker(DePersonNameBrafBroker.instance());
    return getBroker();
  }
}
