package gov.nist.nlpir.irf.de;

import gov.nist.nlpir.irf.de.DeString;
import gov.nist.nlpir.irf.de.DataElem;
import gov.nist.nlpir.irf.document.ProxyDocument;
import gov.nist.nlpir.irf.modality.IndexingModalities;

import java.util.Vector;
import gov.nist.nlpir.irf.broker.PersistenceBroker;
import gov.nist.nlpir.irf.broker.braf.DeStringBrafBroker;

/** 
 * An instance of this class is a proxy for a DeString object.<br><br>
 * 
 * @version $Revision: 1.2 $ 	
 * 
 * @author
 * This software was produced by NIST, an agency of the U.S. government,  
 * and by statute is not subject to copyright in the United States.        
 * Recipients of this software assume all responsibilities associated      
 * with its operation, modification and maintenance.  
 */
public class ProxyDeString extends ProxyDataElem {
      

  /** No-argument constructor */
  public ProxyDeString() {
  }



  /**
   * Creates a proxy for the given DeString given a DeString
   */
  public ProxyDeString(DeString s) {
    setRealObject(s);
  }


  
  /**
   * Creates a DeString with the given String and associates a new proxy 
   * to this DE.
   */
  public ProxyDeString(String s) {
    setRealObject(new DeString(s));
  }
  


  /**
   * Returns the String contained in the associated DeString.
   */
  public final String getValue() {
    return ((DeString)getRealObject()).getValue();
  }



  /**
   * Sets the String value of the associated DeString.
   */
  public final void setValue(String v) {
    ((DeString)getRealObject()).setValue(v);
  }



  /**
   * Sets the String value of the associated DeString from this char array.
   */
  public final void setValue(char v[]) {
    ((DeString)getRealObject()).setValue(v);
  }

 

  /**
   * Returns the associated DeString's value as a String
   * @return the associated DeString's value as a String
   */
  public final String getValueAsString() {
    return ((DeString)getRealObject()).getValueAsString();
  }



  /**
   * Returns the associated DeString's value as a String
   * @return the associated DeString's value as a String
   */
  public final String toString() {
    return ((DeString)getRealObject()).toString();
  }
  


  /**
   * Writes the associated DeString to standard out
   */
  public void present() {
    ((DeString)getRealObject()).present();
  }



  /**
   * Writes the associated DeString to standard out in a brief form
   */
  public void presentBrief() {
    ((DeString)getRealObject()).present();
  }



  /**
   * Compares the associated DeString to another.
   * @param de the DeString to compare to
   * @return true if both represented Strings are the same,<br>
   * false otherwise or if de is not a DeString.
   */
  public boolean exactMatch(DataElem de) {
    return ((DeString)getRealObject()).exactMatch(de);
  }



  /**
   * Scores the match of the real DeString with another
   * For the function to succeed, the object must be a DeString also. 
   * Actually the result is 1 if the two objects exactly match, ie the 
   * Strings they represent are exactly the same, 0 otherwise.
   * @param de the DeString to compared to
   * @return the score of the matching process.
   */
  public float match(DataElem de) {
    return ((DeString)getRealObject()).match(de);
  }



  /**
   * Scores the comparison between the real DeString and the given DeString
   * @return the score as an int: the result is the comparison between the o2
   * String value and the value of this. Returns Integer.MAX_VALUE if o2 is
   * not a DeString.
   * @param o the DeString to compare to
   * @see String#compareTo
   */
  public int compareTo(DataElem o) {
    return ((DeString)getRealObject()).compareTo(o);
  }



  /**
   * Computes a hash code for the real DE_string based on the hashCode of
   * the represented String.
   * @return hash code for use in java.util.Hashtable.
   */
  public int hashCode() {
    return ((DeString)getRealObject()).hashCode();
  }



  /**
   * Extracts Indexing Features from the real DeString.
   * @param source object reference of source document.
   * @param im indexing modalities to be used with this DeString.
   * @param field the document field from which this DE came
   * @return vector of IndexingFeatures.
   */
  public Vector getIndexingFeatures(ProxyDocument source, 
				    IndexingModalities im,
				    DataElem field) {
    return ((DeString)getRealObject()).getIndexingFeatures(source,im,field);
  }



  /**
   *  Adds the string to the HTML text given. If it is to be read
   *  in a form, the HTML code will allow the entry of values.
   *  @param baseString the HTML text the value will be appended to.
   *  @param maxNumberOfChar maximum number of characters added, 
   *  used only if form is false.
   *  @param form true if the HTML code must be used in a form, 
   *  false otherwise.
   *  @return the new HTML string.
   */
  public void addHTML(StringBuffer baseString, int maxNumberOfChar,
		      boolean form){
    ((DeString)getRealObject()).addHTML(baseString, maxNumberOfChar, form);
  }
  


  /**
   *  Adds the value of the string between H1 tags to the given string.
   */
  public void HTML_ShowDoc(StringBuffer baseString) {
    ((DeString)getRealObject()).HTML_ShowDoc(baseString);
  }



  /** 
   * Creates a  broker for this proxy object and remember it
   * @return the broker for this object
   */          
  public PersistenceBroker createBroker(){
    setBroker(DeStringBrafBroker.instance());
    return(getBroker());
  }
}


