package gov.nist.nlpir.irf.de;
import java.lang.Integer;

/**
   This class manages statistics about texts, like the number of words in
   each text, the number of stopwords, etc.
   
   @version $Revision: 1.1 $ 	
   @author
   This software was produced by NIST, an agency of the U.S. government,  
   and by statute is not subject to copyright in the United States.        
   Recipients of this software assume all responsibilities associated      
   with its operation, modification and maintenance.  
*/
public class TextStatistics {
    
    private int totalWords;
    private int totalStopWords;
    private int numberOfDocs;
    private int maxNumberOfWords;
    private int minNumberOfWords;
    private int maxNumberOfStopWords;
    private int minNumberOfStopWords;
    private int docWordCount;
    private int docStopWordCount;
    
    /**
       Creates an object with all statistics being 0, excepts maximums
       that are positionned to Integer.MAX_VALUE so that their computing will be
       easier.
    */
    public TextStatistics() {
	totalWords = 0;
	totalStopWords = 0;
	numberOfDocs = 0;
	maxNumberOfWords = 0;
	minNumberOfWords = Integer.MAX_VALUE;
	maxNumberOfStopWords = 0;
	minNumberOfStopWords = Integer.MAX_VALUE;
	docWordCount = docStopWordCount = 0;
    }

    /**
       This feature must be called when a new stopword is met in a
       document.
    */
    public void newStopWord() {
	totalStopWords++;
	docStopWordCount++;
    }

    /**
       This feature must be called when a new word is met in a document.
    */
    public void newWord() {
	totalWords++;
	docWordCount++;
    }

    /**
       This function must be called when a document has been parsed so
       that the statistics can be updated.
    */
    public void newDoc() {
	numberOfDocs++;
	if (docWordCount > maxNumberOfWords)     maxNumberOfWords = docWordCount;
	if (docWordCount < minNumberOfWords)     minNumberOfWords = docWordCount;
	if (docStopWordCount > maxNumberOfStopWords) maxNumberOfStopWords = docStopWordCount;
	if (docStopWordCount < minNumberOfStopWords) minNumberOfStopWords = docStopWordCount;
	docWordCount = 0;
	docStopWordCount = 0;
    }

    /**
       This function prints all the statistics about the texts, on an eight
       lines display.
    */
    public void present() {
	System.out.println("Number of documents:        " + numberOfDocs );
	System.out.println("Total number of words:      " + totalWords );
	System.out.println("Total number of stop words: " + totalStopWords );
	System.out.println("Percent of stopWords:       " +
			   (float)totalStopWords / (float)totalWords * 100);
	System.out.println("Max number of words:        " + maxNumberOfWords );
	System.out.println("Min number of words:        " + minNumberOfWords );
	System.out.println("Max number of stop words:   " + maxNumberOfStopWords );
	System.out.println("Min number of stop words:   " + minNumberOfStopWords );
    }
}
