package gov.nist.nlpir.irf.document;

import gov.nist.nlpir.irf.modality.IndexingModalities;
import gov.nist.nlpir.irf.modality.IndexingModalityUnit;
import gov.nist.nlpir.irf.modality.RetrievalModalities;

import gov.nist.nlpir.irf.query.QueryRep;
import gov.nist.nlpir.irf.result.RetrievalResult;
import gov.nist.nlpir.irf.handle.Handle;

import java.util.Vector;

/**
 * Parent class of all application IR document interfaces.
 * <br><br>
 * Used to provide/enforce a common view e.g., of Documents and
 * their proxies
 *
 * @version $Revision: 1.2 $
 *
 * @author
 * This software was produced by NIST, an agency of the
 * U.S. government, and by statute is not subject to copyright in the
 * United States.  Recipients of this software assume all
 * responsibilities associated with its operation, modification and
 * maintenance.
 *
 */

public interface DocumentIntrf  {

    
    /** Gets feature names of document.
     * @return The collection of feature names
     */
    public Vector getFeatureNames();
    

    /**
     * Gets feature types of document. 
     * @return A collection of feature types as Strings.
     */
    public Vector getFeatureTypes();
    

    /**
     * Gets the values of the indexable features. Features can be indexed
     * if they are declared public.
     *
     * @return a collection of Data Elements (DataElem)
     */
    public Vector getFeatureValues();


    /**
     * Displays document in ascii form to standard out
     */
    public void present();


    /**
     * Displays document on one line to standard out
     */
    public void presentBrief();
    

    /**
     * Indexes this document.
     *
     * @param im Indexing Modality list .
     */	
    public void index(IndexingModalities im, ProxyDocument proxyDoc);


    /**
     * Returns this document as a query.
     *
     * @param rm Retrieval Modalities
     * @return The retrieval result for the given modalities.
     */
    public RetrievalResult retrieve(RetrievalModalities rm, 
					  ProxyDocument proxyDoc);


    /**
     * Returns indexing features as a Vector of Vectors: one inner
     * Vector for each modality unit.
     *
     * @param im Indexing Modalities
     * @return Vector containing vectors of indexing features
     */
    public Vector getIndexingFeatures(IndexingModalities im, 
				      ProxyDocument peroxyDoc);


    /**
     * Gets indexing modalities for this document.
     * should be overriden by subclass.
     * @return null always.
     */
    public IndexingModalities getIndexingModalities();


    /**
     * Gets default indexing modalities for this document.
     * @return Indexing modalities for this document.
     *
    public IndexingModalities getDefaultIndexingModalities();
    */ // remove


    /** 
     * Adds HTML to the given string buffer.
     *
     * @param baseString The StringBuffer the HTML text will be appended to.
     * @param unused Unused.
     * @param form Boolean set to true if you wish HTML document to be
     * a form document.
     */
    public void addHTML(StringBuffer baseString, int unused, boolean form);


    /**
     * Makes a complete result for documents matching the supplied
     * query from the partial results supplied.
     *
     * @param query The  query to match.
     * @param collPR A collection of partial Results.
     * @return A Vector of ResultForDocMatchingQuery. There is one of them 
     * by Document in the results, containing every Result..ModalityUnit 
     * linked to this document.
     */
    public Vector makeResultsForDocMatchingQuery(QueryRep query, 
						 Vector collPartialResults);

}
