package gov.nist.nlpir.irf.feature;

import gov.nist.nlpir.irf.feature.IndexingFeature;
import gov.nist.nlpir.irf.document.ProxyDocument;
import gov.nist.nlpir.irf.de.ProxyDataElem;

/**
 * This particular IndexingFeature allows you to keep track of the positional
 * information of a word within a String with the variable wordCount. It will
 * then allow finer grain research and display.
 *
 * @version $Revision: 1.1 $
 *
 * @author
 * This software was produced by NIST, an agency of the U.S. government,
 * and by statute is not subject to copyright in the United States.
 * Recipients of this software assume all responsibilities associated with
 * its operation, modification and maintenance.
 */

public class IdxFeatWordInString extends IndexingFeature {

  /** The rank of the word in the String. */
  private int wordCount;    

  /**
   * @param f feature to be stored
   * @param s source document to be stored
   */

  public IdxFeatWordInString(ProxyDataElem feature, ProxyDocument sourceDoc,
			 ProxyDataElem sourceField, int wordCount) {
    super(feature, sourceDoc, sourceField);
    this.wordCount = wordCount;
  }

  /**
   * returns string representation of indexing feature showing
   * feature and source.
   * @return string representation of indexing feature
   */

  public String toString() {
    return super.toString() + "wC: " + wordCount;
  }

  /**
   * Gets the positional information. 
   */

  public final int getWordCount() {
    return wordCount;
  }

  /**
   * Sets the postional information.
   */

  public final void setWordCount(int wordCount) {
    this.wordCount = wordCount;
  }
}
