package gov.nist.nlpir.irf.feature;

import gov.nist.nlpir.irf.document.ProxyDocument;
import gov.nist.nlpir.irf.de.ProxyDataElem;

/**
 * Container for elements of an Indexing Feature, the feature
 * itself (a term, vector, label, etc.) and the source document
 * that it was generated from.
 *     
 * @version $Revision: 1.1 $
 *
 * @author
 * This software was produced by NIST, an agency of the U.S. government,
 * and by statute is not subject to copyright in the United States.
 * Recipients of this software assume all responsibilities associated with
 * its operation, modification and maintenance.
 */

public class IdxFeatWordInText extends IdxFeatWordInString {

  /** Paragraph number information. */
  private int paragraphCount;    

  /**
   * @param f feature to be stored
   * @param s source document to be stored
   */

  public IdxFeatWordInText(ProxyDataElem feature, ProxyDocument sourceDoc,
		       ProxyDataElem sourceField, int wordCount, int paragraphCount) {
    super(feature, sourceDoc, sourceField, wordCount);
    this.paragraphCount = paragraphCount;
  }

  /**
   * returns string representation of indexing feature showing
   * feature and source.
   * @return string representation of indexing feature
   */

  public String toString() {
    return super.toString() + "pC: " + paragraphCount;
  }

  /**
   * Gets the paragraph number information.
   */

  public final int getParagraphCount() {
    return paragraphCount;
  }

  /**
   * Sets the paragraph number information.
   */

  public final void setParagraphCount(int paragraphCount) {
    this.paragraphCount = paragraphCount;
  }
}
