package gov.nist.nlpir.irf.feature;

import gov.nist.nlpir.irf.document.ProxyDocument;
import gov.nist.nlpir.irf.de.ProxyDataElem;
import gov.nist.nlpir.irf.broker.PersistenceBroker;
import gov.nist.nlpir.irf.broker.braf.IdxFeatStringBrafBroker;
import gov.nist.nlpir.irf.util.Support;
import gov.nist.nlpir.irf.feature.IndexingFeature;

/**
 * Proxy class for Indexing Features referring to a word in a string. This particular
 * IndexingFeature allows you to keep track of the positional information of this word
 * with the variable wordCount. It will then allow finer grain research and display.
 *
 * @version $Revision: 1.1 $ 
 *
 * @author
 * This software was produced by NIST, an agency of the U.S. government,
 * and by statute is not subject to copyright in the United States.
 * Recipients of this software assume all responsibilities associated with
 * its operation, modification and maintenance.
 *
 * @see IdxFeatWordInString
 */

public class ProxyIdxFeatWordInString extends ProxyIndexingFeature {

  /**
   * @param feature Feature to be stored.
   * @param sourceDoc Source document to be stored.
   * @param sourceField Field in the document from where this feature comes.
   * @param wordCount position of this word in the String it is extracted from.
   */
  public ProxyIdxFeatWordInString(ProxyDataElem feature, ProxyDocument sourceDoc,
			      ProxyDataElem sourceField, int wordCount) {
    setRealObject(new IdxFeatWordInString(feature, sourceDoc, sourceField, wordCount));
  }
  
  /**
   * No argument constructor for deserialization purpose.
   */
  public ProxyIdxFeatWordInString() {
  }

  /**
   * Access to the position information. 
   */  
  public final int getWordCount() {
    return ((IdxFeatWordInString)getRealObject()).getWordCount();
  }

  /**
   * Sets the position information.
   */
  public final void setWordCount(int wordCount) {
    ((IdxFeatWordInString)getRealObject()).setWordCount(wordCount);
  }

  /**
   * The concrete proxy will override this method to get the broker
   * for the proxy object @return the broker for this object 
   */
  public PersistenceBroker createBroker() {
    /* alternative for this is to have one broker stored in the InfoServer
       and retrieved in this method */
    return IdxFeatStringBrafBroker.instance();
  }
  
  /**
   * Writes the IdxFeatWordInString preceded by <code>P </code>.
   */
  public String toString() {
    return super.toString() + " wC=" + getWordCount();
  }
}
