package gov.nist.nlpir.irf.feature;

import gov.nist.nlpir.irf.document.ProxyDocument;
import gov.nist.nlpir.irf.de.ProxyDataElem;
import gov.nist.nlpir.irf.util.Support;
import gov.nist.nlpir.irf.broker.PersistenceBroker;
import gov.nist.nlpir.irf.broker.braf.IdxFeatTextBrafBroker;
/**
 * Proxy class for Indexing Features referring to a word in a text.
 *
 * @version $Revision: 1.1 $ 
 *
 * @author
 * This software was produced by NIST, an agency of the U.S. government, and by statute
 * is not subject to copyright in the United States.  Recipients of this software assume
 * all responsibilities associated with its operation, modification and maintenance.
 *
 * @see IdxFeatWordInText
 */

public class ProxyIdxFeatWordInText extends ProxyIdxFeatWordInString {

  /**
   * @param feature Feature to be stored.
   * @param sourceDoc Source document to be stored.
   * @param sourceField Field in the document from where this feature comes.
   * @param wordCount position of this word in the paragraph it is extracted from.
   * @param paragraphCount number of the paragraph this feature is in (in the
   * sourceField argument actually).
   */
  public ProxyIdxFeatWordInText(ProxyDataElem feature, ProxyDocument sourceDoc,
			    ProxyDataElem sourceField, int wordCount, int paragraphCount) {
    setRealObject(new IdxFeatWordInText(feature, sourceDoc, sourceField,
				    wordCount, paragraphCount));
  }
  
  /**
   * Empty constructor for deserializatiopn purpose.
   */
  public ProxyIdxFeatWordInText() {
  }
  
  /**
   * Gets the positional information (paragraph number).
   */
  public final int getParagraphCount() {
    return ((IdxFeatWordInText)getRealObject()).getParagraphCount();
  }

  /**
   * Sets the positional information (paragraph number).
   */
  public final void setParagraphCount(int paragraphCount) {
    ((IdxFeatWordInText)getRealObject()).setParagraphCount(paragraphCount);
  }

  /**
   * The concrete proxy will override this method to get the broker
   * for the proxy object @return the broker for this object 
   */
  public PersistenceBroker createBroker() {
    /* alternative for this is to have one broker stored in the InfoServer
       and retrieved in this method */
    return IdxFeatTextBrafBroker.instance();
  }

  /**
   * Display method, returning the IdxFeatWordInText preceded by <code>P </code>.
   */
  public String toString() {
    return super.toString() + " pC=" + getParagraphCount();
  }
}
