package gov.nist.nlpir.irf.handle;

import gov.nist.nlpir.misc.io.BufferedRandomAccessFile;

/**
 * Parent class of all handles - objects which provide information of various
 * sorts defined and needed by a broker to access a stored object. 
 * <br><br>
 * Only brokers need know the form and content of the handles they use.
 *
 * @version $Revision: 1.1 $
 *
 * @author
 * This software was produced by NIST, an agency of the U.S. government,
 * and by statute is not subject to copyright in the United States.
 * Recipients of this software assume all responsibilities associated
 * with its operation, modification and maintenance.
 */
abstract public class Handle {


  /**
   * Writes an external representation of this object to a file
   * @param out buffered random access file handle
   * @return The size of the handle in the storage file.
   */
  abstract public void writeExt(BufferedRandomAccessFile out);



  /**
   * Replaces the existing handle in the given file if it is possible at 
   * this place.
   */
  abstract public boolean replaceExt(BufferedRandomAccessFile out);



  /*
   * Reads an external representation of this object from a file
   * @param in buffered random access file handle
   */
  abstract public void readExt(BufferedRandomAccessFile in);
}
