package gov.nist.nlpir.irf.handle.braf;

import gov.nist.nlpir.irf.handle.Handle;
import java.io.Serializable;
import java.io.IOException;
import gov.nist.nlpir.misc.io.BufferedRandomAccessFile;

/**
 * Handle for a Feature List object brokered by a random access file broker
 *
 * @version $Revision: 1.4 $ 
 *
 * @author
 * This software was produced by NIST, an agency of the
 * U.S. government, and by statute is not subject to copyright in the
 * United States.  Recipients of this software assume all
 * responsibilities associated with its operation, modification and
 * maintenance.
 */

public class FeatureListBrafHandle extends Handle implements 
Serializable {


  /** serial version universal id - put here so Java does not insert one 
   * which may change due to revisions and make it impossible to deserialize
   * earlier versions of serialized objects */
  static final long serialVersionUID = 0;


  /** This value is actually the offset of the first feature stored in
   * the feature list.
   * @serial  */
  private long value;


  /** Byte offset within the file of the link field within the last
   * feature in the list.
   * @serial */
  private long offsetToLink = gov.nist.nlpir.irf.index.braf.LinkedListPool.END_MARKER;

  /** A number given to each pool file to distinguish it from others
      @serial */
  private int filePoolCode;


  /** 
   * Constructs a new handle given a value
   * @param newValue new value for handle 
   */
  public FeatureListBrafHandle(String poolFileName, long newValue) {
    value = newValue;
    filePoolCode = BufferedRandomAccessFileHandle.getCode(poolFileName);
  }



  /** Constructs an instance of a FeatureListBrafHandle from no 
   * arguments
   */
  public FeatureListBrafHandle() {
  }



  /**
   * Sets value of handle 
   * @param newValue new value for handle
   */
  public void setValue(long newValue) {
    value = newValue;
  }



  /**
   * Returns value of handle
   * @return value of handle
   */
  public long getValue() {
    return value;
  }



  /**
   * Sets value of the offsetToLink
   * @param newOne new value for offsetToLink
   */
  public void setOffsetToLink(long newOne) {
    offsetToLink = newOne;
  }
  


  /**
   * Returns value of offsetToLink
   * @return value of offsetToLink
   */
  public long getOffsetToLink() {
    return offsetToLink;
  }



  /**
   * Returns a string representation of the handle
   * @return string representation of handle
   */
  public String toString() {
    return "FeatureList Handle: " + value;
  }
  


  /**
   * Reads external representation of this object
   * @param in buffered random access file handle
   */
  public final void readExt(BufferedRandomAccessFile in) {
    try {
      filePoolCode = in.readByte();
      value = in.readLong();
      offsetToLink = in.readLong();
    } catch (IOException e) {
      value = 0;
    }
  }



  /**
   * Writes external representation of this object
   * @param out buffered random access file handle
   */
  public final void writeExt(BufferedRandomAccessFile out) {
    try {
      out.writeByte(filePoolCode);
      out.writeLong(value);
      out.writeLong(offsetToLink);
    } catch (IOException e) {
      System.err.println("Error writing long value: " + e);
    }
  }



  /**
   * Writes external representation of this object
   * @param out buffered random access file handle
   */
  public final boolean replaceExt(BufferedRandomAccessFile out) {
    try {
      out.writeByte(filePoolCode);
      out.writeLong(value);
      out.writeLong(offsetToLink);
      return true;
    } catch (IOException e) {
      System.err.println("Error writing long value: " + e);
    }
    return false;
  }


  /**
   * Returns the pool file number so that the broker
   * knows which LinkedListPool it must talk to.
   */
  public final int getFilePoolCode(){
    return filePoolCode;
  }
}
