package gov.nist.nlpir.irf.index;

import gov.nist.nlpir.irf.query.QueryModalityUnit;

import gov.nist.nlpir.irf.indexer.Indexer;

import java.util.Enumeration;
import java.util.Vector;

/** 
 * This interface represents the functionality of a Index object.
 * <br><br>
 * 
 * @version $Revision: 1.2 $
 * 
 * @author
 * This software was produced by NIST, an agency of the U.S. government,  
 * and by statute is not subject to copyright in the United States.        
 * Recipients of this software assume all responsibilities associated      
 * with its operation, modification and maintenance.  
 *                             
 * @see Index
 * @see ProxyIndex
 */

public interface IndexIntrf {    
    /** 
     * Gets the index name.
     * 
     * @return The index name.
     */
    public String getName();
    
    /**
     * Returns the features for which indexing features have been stored.
     * @return The Enumeration of Objects (should be DeIntern) used as
     * features in this Index
     */
    public Enumeration getFeatures();
    
    /**
     * Returns the sources for which indexing features have been stored.
     * @return The Enumeration of Objects (should be IoAddrIntern) used as
     * sources in this Index
     */
    public Enumeration getSources();

    /** 
     * Updates the index and its features.
     */
    public void update();
    
    /** 
     * Clears the index and its features.
     */
    public void clear();
    
    /** 
     * Executes a search and retrieves documents.
     * 
     * @param qm Query Modality Unit
     * @param setIF Vector of Vectors of indexing features
     * @return Results or null if submitted to the wrong index.
     */
    public Vector retrieve(QueryModalityUnit qm, Vector setIF);
    
    /** 
     *Presents the status of the index and its features.
     */
    public void present();
    
    /** 
     * Presents the status of the index and its feature statistics.
     */
    public void presentStatistics();
    
    /**
     * Gives a String Representaiton of the Index.
     */
    public String toString();
    
    /** 
     * Verifies that the same indexer is applied consistantly.
     * 
     * @param an_Indexer an indexer
     * @return true if the Index is well applied, false otherwise.
     */
    
    public boolean checkIndexer(Indexer anIndexer);
    
    /** 
     * Add indexing features.
     * 
     * @param indexingFeatures Vector of indexing features.
     * @param anIndexer an indexer.
     * @return true if the Indexer exists and is correct,<br>
     * false otherwise
     */
    
    public boolean addIndexingFeatures(Vector indexingFeatures,
				       Indexer anIndexer);
    
    /**
     * statistics about the index.
     *
     * @param deep deepness of the statistics in the intern index
     * @param maxLengthIf3 maximum number of characters printed.
     * @see gov.nist.nlpir.irf.index.braf.PersistentDualKeyContainer#showStatistics
     */

    public void showStatistics(int depth, int maxLengthIf3);
}
