package gov.nist.nlpir.irf.index;

import gov.nist.nlpir.irf.result.ResultForDocMatchingQueryModalityUnit;
import gov.nist.nlpir.irf.result.ResultForDocMatchingQueryCondition;
import gov.nist.nlpir.irf.index.braf.DeIntern;

import java.util.Enumeration;

/** 
 * Extends the IdxIntern for KeyWord indexes.
 *
 * @version $Revision: 1.3 $
 * 
 * @author
 * This software was produced by NIST, an agency of the U.S. government,  
 * and by statute is not subject to copyright in the United States.        
 * Recipients of this software assume all responsibilities associated      
 * with its operation, modification and maintenance.                          
 */

public class KeyWordIdxIntern extends IdxIntern {
    

  public KeyWordIdxIntern(String loc, String name) {
    super(loc,name);
  }



  /** 
   * Calculates the retrieval status value.
   * 
   * @param a_Result a result.
   * @return return status value
   */
  public double calcRSV(ResultForDocMatchingQueryModalityUnit aResult) {
    double RSV;
    int i = aResult.getNumUniqueQueryFeatures(); // Num key words in query
    int j = 0; // Number of the query key words found in document
    Enumeration elmEnum = aResult.getPartialResults().elements();
    ResultForDocMatchingQueryCondition aPR;
    while (elmEnum.hasMoreElements()) {
      aPR = (ResultForDocMatchingQueryCondition)elmEnum.nextElement();
      if (!aPR.getBasicResults().isEmpty())
	j++;
    }	
    RSV = (double)j / i;
    aResult.setRSV(RSV);
    return RSV;
  }
    


  /** 
   * Displays an ascii representation of the features.
   */
  public void presentFeatures() {
    Enumeration c1 = getFeatures();
    while (c1.hasMoreElements() ) {
      DeIntern an_DE = (DeIntern)c1.nextElement();
      System.out.println(an_DE.getValueAsString());
    }
  }



  /** 
   * Displays an ascii representation of the Class.
   * Currently empty but required to realize this concrete class from its 
   * abstract parent. 
   */
  public void present() {}
    


  /** 
   * Displays an ascii representation of the sources.
   * Currently empty but required to realize this concrete class from its 
   * abstract parent. 
   */
  public void presentSources() {}
    


  /**
   * Currently does nothing.
   */
  public void calcInputForRSV(ResultForDocMatchingQueryCondition r) {}

}
