package gov.nist.nlpir.irf.index;

import gov.nist.nlpir.irf.index.braf.IoAddrIntern;
import gov.nist.nlpir.irf.index.braf.DeIntern;
import gov.nist.nlpir.irf.query.QueryModalityUnit;
import gov.nist.nlpir.irf.proxy.VirtualProxy;
import gov.nist.nlpir.irf.broker.PersistenceBroker;
import gov.nist.nlpir.irf.broker.braf.IndexBrafBroker;
import gov.nist.nlpir.irf.document.ProxyDocument;
import gov.nist.nlpir.irf.de.ProxyDataElem;
import gov.nist.nlpir.irf.indexer.Indexer;
import java.util.Vector;
import java.util.Enumeration;

/** 
 * This class represents a proxy for Index objects
 * 
 * @version $Revision: 1.3 $
 * 
 * @author
 * This software was produced by NIST, an agency of the U.S. government,  
 * and by statute is not subject to copyright in the United States.        
 * Recipients of this software assume all responsibilities associated      
 * with its operation, modification and maintenance.  
 *                             
 * @see Index
 */

public abstract class ProxyIndex extends VirtualProxy implements IndexIntrf {
  
  /** 
   * Gets the index name.
   * 
   * @return The index name.
   */
  public String getName() {
    return(((Index)getRealObject()).getName());
  }
  
  /** 
   * Updates the index and its features.
   */
  public void update() {
    ((Index)getRealObject()).update();
  }
  
  /** 
   * Clears the index and its features.
   */
  public void clear() {
    ((Index)getRealObject()).clear();
  }
  
  /** 
   * Executes a search and retrieves documents.
   * 
   * @param qm Query Modality Unit
   * @param setIF Vector of Vectors of indexing features
   * @return Results or null if submitted to the wrong index.
   */
  public Vector retrieve(QueryModalityUnit qm, Vector setIF) {
    return(((Index)getRealObject()).retrieve(qm, setIF));
  }
  
  /** 
   *Presents the status of the index and its features.
   */
  public void present() {
    ((Index)getRealObject()).present();
  }
  
  /** 
   * Presents the status of the index and its feature statistics.
   */
  public void presentStatistics() {
    ((Index)getRealObject()).presentStatistics();
  }
  
  /**
   * Gives a String Representation of the Index.
   */
  public String toString() {
    return(((Index)getRealObject()).toString());
  }
  
  /** 
   * Verifies that the same indexer is applied consistantly.
   * 
   * @param an_Indexer an indexer
   * @return true if the Index is well applied, false otherwise.
   */
  public boolean checkIndexer(Indexer anIndexer) {
    return(((Index)getRealObject()).checkIndexer(anIndexer));
  }
  
  /** 
   * Add indexing features.
   * 
   * @param indexingFeatures Vector of indexing features.
   * @param anIndexer an indexer.
   * @return true if the Indexer exists and is correct,<br>
   * false otherwise
   */
  public boolean addIndexingFeatures(Vector indexingFeatures, Indexer anIndexer) {
    return ((Index)getRealObject()).addIndexingFeatures(indexingFeatures, anIndexer);
  }
  
  /**
   * statistics about the index.
   *
   * @param depth Depth of the statistics in the intern index.
   * @param maxLengthIf3 Maximum number of characters printed.
   * @see gov.nist.nlpir.irf.index.braf.PersistentDualKeyContainer#showStatistics
   */
  
  public void showStatistics(int depth, int maxLengthIf3) {
    ((Index)getRealObject()).showStatistics(depth, maxLengthIf3);
  }
  
  /** 
   * Create a  broker for this proxy object and remember it
   * @return the broker for this object
   */          
  public PersistenceBroker createBroker(){
    setBroker(IndexBrafBroker.instance());
    return(getBroker());
  }

  /**
   * @see Index#getSources
   */  
  public Enumeration getSources() {
    return ((Index)getRealObject()).getSources();
  }
  
  /**
   * @see Index#getFeatures
   */  
  public Enumeration getFeatures() {
    return ((Index)getRealObject()).getFeatures();
  }
  
  /**
   * Makes the real object persistent. This proxy now has a goal. Recursively calls
   * makePersistent on every source and every feature contained in the index so that
   * it is consistent.
   */  
  public void makePersistent() {
    if (objectIdentifier == null) { // Only needed if real object not yet persistent
      
      Enumeration sources = getSources();
      while (sources.hasMoreElements()) {
	ProxyDocument doc =  ((IoAddrIntern)sources.nextElement()).getDoc();
	doc.makePersistent();
      }
      
      Enumeration features = getFeatures();
      while (features.hasMoreElements()) {
	ProxyDataElem field =  (ProxyDataElem)((DeIntern)features.nextElement()).getDE();
	field.makePersistent();
      }

      super.makePersistent();
    }    
  }
}
