package gov.nist.nlpir.irf.index.braf;

/**
 * This class extends Vector so that it can fit our needs for
 * persistence, and mainly the management of references to proxies.
 * All three constructors have the exact same meaning than in Vector.
 *
 * @version $Revision: 1.4 $ 
 *
 * @author
 * This software was produced by NIST, an agency of the U.S. government,
 * and by statute is not subject to copyright in the United States.
 * Recipients of this software assume all responsibilities associated
 * with its operation, modification and maintenance.
 *
 * @see gov.nist.nlpir.irf.index.DualKeyContainer
 */

public class FeatureList extends java.util.Vector {

  /** @serial flag indicating refs to proxies must be counted */
  private boolean proxyRefsCountedTrue = false;



  public FeatureList() {
    super();
  }


  public FeatureList(int capacity) {
    super(capacity);
  }


  public FeatureList(int capacity, int capacityIncrement) {
    super(capacity, capacityIncrement);
  }



  /**
   * This method is "persistence-ready" unlike the one in the super class.
   */
  public synchronized Object clone() {
    if (proxyRefsCountedTrue) {
      int size = elementData.length;
      for (int i = 0; i < size; i++) {
	Object curr = elementData[i];
	if (curr != null)
	  ((gov.nist.nlpir.irf.proxy.VirtualProxy)curr).addRefToInMemoryProxiesByOid();
      }
    }
    return (FeatureList)super.clone();
  }
  
  

  /**
   * This method must be called if this FeatureList has registered its
   * references.
   */
  public void setProxyRefsCountedTrue() {
    proxyRefsCountedTrue = true;
  }



  /**
   * Special definition of the method so that it doesn't take up too
   * much space, mainly because IndexingFeatures are huge to display.
   */
  public String toString() {
    String result = "[";
    int size = elementData.length;
    for (int i = 0; i < size; i++)
      if (elementData[i] != null) {
	String toAdd = elementData[i].toString();
	if (toAdd.length() > 30)
	  toAdd = toAdd.substring(0, 29);
	result += toAdd + ", ";
      } else
	result += "null, ";
    result += "]";
    return result;
  }  


  /**
   * "persistence-ready" version of the default method.
   */
  public void finalize() {
    if (proxyRefsCountedTrue) {
      int size = elementData.length;
      for (int i = 0; i < size; i++) {
	Object curr = elementData[i];
	if (curr != null)
	  ((gov.nist.nlpir.irf.proxy.VirtualProxy)curr).deleteRefFromInMemoryProxiesByOid();
      }
    }
  }
}
