package gov.nist.nlpir.irf.indexer;

import gov.nist.nlpir.irf.de.DataElem;
import gov.nist.nlpir.irf.document.ProxyDocument;
import java.io.IOException;
import java.io.FileNotFoundException;
import java.util.Vector;

/** 
 * The topmost abstract class for all indexer classes.
 * <br><br>
 * 
 * @version $Revision: 1.2 $ 	
 * 
 * @author
 * This software was produced by NIST, an agency of the U.S. government,  
 * and by statute is not subject to copyright in the United States.        
 * Recipients of this software assume all responsibilities associated      
 * with its operation, modification and maintenance.                             
 */                      

public abstract class Indexer implements java.io.Serializable {


  /** serial version universal id - put here so Java does not insert one 
   * which may change due to revisions and make it impossible to deserialize
   * earlier versions of serialized objects */
  static final long serialVersionUID = 0;

  /**
   * All indexer classes must implement this method to return the indexing
   * features that were extracted from specified data element and document
   * at indexing time.
   *
   * @param de the data element from which to extract features
   * @param source the document which contains the data element
   * @return a vector of features for the specified data element and document
   */          
    public abstract Vector getIndexingFeatures(DataElem de, 
					       ProxyDocument source); 
}

