package gov.nist.nlpir.irf.modality;

import gov.nist.nlpir.irf.document.DocumentIntrf;
import gov.nist.nlpir.irf.modality.IndexingModalityUnit;

import java.util.Vector;

/**
 * A collection of Indexing Modality Units
 * <br><br>      
 * @version $Revision: 1.2 $
 *
 * @author
 * This software was produced by NIST, an agency of the
 * U.S. government, and by statute is not subject to copyright in the
 * United States.  Recipients of this software assume all
 * responsibilities associated with its operation, modification and
 * maintenance.
 *
 * @see gov.nist.nlpir.irf.modality.IndexingModalityUnit
 * @see gov.nist.nlpir.irf.document.Document
 */

public class IndexingModalities implements java.io.Serializable {


  /** serial version universal id - put here so Java does not insert one 
   * which may change due to revisions and make it impossible to deserialize
   * earlier versions of serialized objects */
  static final long serialVersionUID = 0;


  /** Vector of IndexingModalityUnits 
   * @serial */
  private Vector modalitylist;
    


  /** 
   * Constructs a new modalities container 
   */
  public IndexingModalities() { 
    modalitylist = new Vector(); 
  }
    


  /** 
   * Adds a modality 
   * @param imu indexing modality unit
   */
  final public void addModality(IndexingModalityUnit imu) {
    modalitylist.addElement(imu);
  }



  /** 
   * Gets the count of modalities
   * @return count of modalities
   */
  final public int getModalityCount() {
    return modalitylist.size();
  }   



  /** 
   * Gets modality unit at a given index (0-based)
   * @param index index of IMU to return
   * @return n-th modality
   */
  final public IndexingModalityUnit getModalityAt(int index) {
    return (IndexingModalityUnit)modalitylist.elementAt(index);
  }  



  /** 
   * Removes modality unit at given index (0-based)
   * @param index index of IMU to remove
   */
  final public void removeModalityAt(int index) {
    modalitylist.removeElementAt(index);
  }     



  /** 
   * Returns vector of indexing modality units.
   * @return collection of indexing modalities as a vector. 
   */
  final public Vector getModalities() { return modalitylist; }
    
 

  /** 
   * Displays indexing modalities 
   */   
  public void present() { 
    int modalityNumber = modalitylist.size();
    System.out.println("Modalities: " + modalityNumber);
    if (modalityNumber > 0) {
      System.out.print("(Feature name, Indexer, Index Name)" +
		       "\n---------------------------------------------------------------\n");
      for (int count = 0; count < modalityNumber; count++) {
	IndexingModalityUnit imu = (IndexingModalityUnit)modalitylist.elementAt(count);
	System.out.print((count+1) + ": ");
	imu.present();
      }
      System.out.println("---------------------------------------------------------------");
    }
  }
    


  /** 
   * Validates settings w.r.t. a Doc
   * @param r document to validate against
   * @return true if valid, false otherwise.
   */
  public boolean validate(DocumentIntrf r) { 
    int modalityNumber = modalitylist.size();
    for (int i = 0; i < modalityNumber; i++) {
      IndexingModalityUnit imu = (IndexingModalityUnit)modalitylist.elementAt(i);
      if (!imu.validate(r))
	return false;
    }
    return true;
  }



  /**
   * Saves each IMU
   */
  public void save() {
    int modalityListSize = modalitylist.size();
    for (int i=0; i<modalityListSize; i++)
      {
      ((IndexingModalityUnit)modalitylist.elementAt(i)).save();
      }
  }
  

}
