package gov.nist.nlpir.irf.modality;

import java.util.Vector;
import gov.nist.nlpir.irf.query.Combinator;

/**
 * Container for information needed for retrieval, namely: indexing
 * modalities and the combinator structure used for retrieval, a
 * set of instructions for combining the results of lower-level
 * retrieval results. 
 *     
 * @version $Revision: 1.1 $
 *
 * @author
 * This software was produced by NIST, an agency of the
 * U.S. government, and by statute is not subject to copyright in the
 * United States.  Recipients of this software assume all
 * responsibilities associated with its operation, modification and
 * maintenance.
 *
 * @see IndexingModalities
 */

public class RetrievalModalities {
    
  /** The information that was used to control indexing */
  private IndexingModalities indexingModalities; 

  /** The map for combining lower-level retrieval results */
  private Combinator combinatorStructure;	
    


  /**
   * Constructs retrieval modalities using indexing modalities.
   *
   * @param i The indexing modalities.
   */
  public RetrievalModalities(IndexingModalities i) { 
    indexingModalities = i; 
    combinatorStructure = null;
  }
    


  /**
   * Gets the indexing modalities for these modalities.
   *
   * @return The indexing modalities.
   */
  public IndexingModalities getIndexingModalities() { 
    return indexingModalities;
  }
    


  /**
   * Gets the combinator structure for these modalities.
   *
   * @returnc The combinator structure.
   */
  public Combinator getCombinatorStructure() {
      return combinatorStructure;
  }


  
  /**
   * Sets the combinator structure for these modalities.
   *
   * @param s The combinator structure.
   */
  public void setCombinatorStructure(Combinator s) { 
    combinatorStructure = s;
  }
}

