package gov.nist.nlpir.irf.modality;

/**
 * Container for a Retrieval Modality Unit (RMU) consisting of a featurename
 * and corresponding indexing modality unit. The RMU models information
 * that describes/controls the role a particular document feature (i.e.,
 * field) plays in a retrieval. This role is closely related to its
 * analogue at indexing time: the indexing modality unit.
 *     
 * @version $Revision: 1.1 $
 *
 * @author
 * This software was produced by NIST, an agency of the
 * U.S. government, and by statute is not subject to copyright in the
 * United States.  Recipients of this software assume all
 * responsibilities associated with its operation, modification and
 * maintenance.
 *
 * @see gov.nist.nlpir.irf.modality.IndexingModalityUnit
 */

public class RetrievalModalityUnit extends Object {
    
  /** Name of the feature (i.e., field within a document) associated with 
   *  this  RMU */
  private String featureName;

  /** The indexing modality unit associated with this RMU */
  private IndexingModalityUnit imu;
    


  /**
   * @param f The feature name.
   * @param i The indexing modality unit.
   */
  public RetrievalModalityUnit(String f, IndexingModalityUnit i) {
    featureName = f;
    imu = i;
  }
   

 
  /**
   * Gets the  feature name of this modality unit.
   *
   * @return The feature name as a string.
   */  
  public String getFeatureName() {
	return featureName;
  }
    


  /**
   * Returns a string representation of the RetrievalModalityUnit with
   * its featureName and the corresponding IndexingModalityUnit.
   *
   * @return the string representation of this RetrievalModalityUnit.
   */ 
  public String toString() {
    return "rmu: {featureName: " + featureName +", imu: " + imu + "}";
  }
}
