package gov.nist.nlpir.irf.query;

import gov.nist.nlpir.irf.modality.IndexingModalities;
import gov.nist.nlpir.irf.modality.IndexingModalityUnit;
import java.util.Vector;


/**
 * The result combinator structure.
 *
 * @version $Revision: 1.1 $ 
 *
 * @author
 * This software was produced by NIST, an agency of the
 * U.S. government, and by statute is not subject to copyright in the
 * United States.  Recipients of this software assume all
 * responsibilities associated with its operation, modification and
 * maintenance.
 */


public abstract class Combinator extends Vector {


  Combinator(){
  }



  /**
   * Create a combinator.
   * @param size number of objects to be held by combinator.
   */
  Combinator(int size){
    super(size);
  }



  /**
   * Prints the number of spaces specified in parameter.
   * @param i width in characters of indentation.
   */
    // This function is placed here to avoid replicating the methed in the
    // desendants which use it.
  void printSpaces(int i) {
    for (int j = 0; j < i; j++)
      System.out.print(' ');
  }



  /**
   * Prints combinator indexing modality unit.
   * @param imu the indexing modality unit to be printed.
   */
    // This function is placed here to avoid replicating the methed in the
    // desendants which use it.
  void presentCombinatorIMU(IndexingModalityUnit imu){
    System.out.print(imu.getFeatureName() + "  ");
    System.out.print(imu.getIndexName() + "\n");
  }



  /**
   * Displays combinator structure to standard output.
   * @param i width in characters of indentation
   * @param im current indexing modalities
   */
  public abstract void present(int i, IndexingModalities im);


  /**
   * Displays combinator structure, using indexes instead of imus, to standard
   * output.
   * @param i width in characters of indentation
   */
  public abstract void presentBrief(int i);


  /**
   * Displays the combinator structure v, using indexes instead of imus, to 
   * standard output.
   * @param v a combinator
   * @param i width in characters of indentation
   */
  public abstract void presentBrief(Vector v, int i);


}
