package gov.nist.nlpir.irf.query;

import gov.nist.nlpir.irf.de.DeDouble;
import gov.nist.nlpir.irf.de.DeInteger;
import gov.nist.nlpir.irf.modality.IndexingModalities;
import gov.nist.nlpir.irf.modality.IndexingModalityUnit;

import java.util.Vector;

/**
 * The Linear Combinator
 *
 * @version $Revision: 1.1 $ 
 *
 * @author
 * This software was produced by NIST, an agency of the
 * U.S. government, and by statute is not subject to copyright in the
 * United States.  Recipients of this software assume all
 * responsibilities associated with its operation, modification and
 * maintenance.
 */


public class LinearCombinator extends Combinator {

    public LinearCombinator(){
    }

  /**
   * Create a linear combinator.
   * @param size number of objects to be held by the linear combinator.
   */
    public LinearCombinator(int size){
	super(size);
    }

  /**
   * Create and set linear combinator.
   *
   * @param im current indexing modalities
   * @param normalized weights used in scoring
   */
    public LinearCombinator(int numberOfModalities,
			    double[] normalizedWeights){
	
	// JT, 8/18/99: The size of the Vector, combinator, is known in this
	// case - since eachtwo elements are required for each
	// IndexingModality.
	// Thus, initialize combinator to twice the number of
	// IndexingModalities and avoid the need to autoincrement for the
	// second half of the Vector's construction.

	this(numberOfModalities*2);
	for (int i = 0; i < numberOfModalities; i++) {
	    addElement(new DeInteger(i));                   // IMU index.
	    addElement(new DeDouble(normalizedWeights[i])); // IMU weight.
	}
    }


  /**
   * display current linear combinator to standard output.
   * @param i width in characters of indentation
   * @param im current indexing modalities
   */
  
  
    public void present(int i, IndexingModalities im) {
	int s  = size();
	int j = 1;
	System.out.print("Linear combination, equal weights\n");
	while (j < s) {
	    System.out.print("   " +
			     ((DeDouble)(elementAt(j))).getValue() + " : ");
	    presentCombinatorIMU((IndexingModalityUnit)
				 (im.getModalities().elementAt(((DeInteger)(elementAt(j-1))).getValue())));
	    j += 2;
	}
    }

  /**
   * display current linear combinator using indexes instead of imus 
   * to standard output.
   * @param i width in characters of indentation
   */
  
  
    public void presentBrief(int i) {
	int s  = size();
	int j = 1;
	while (j < s) {
	    System.out.print("   " +
			     ((DeDouble)(elementAt(j))).getValue() + " : ");
	    System.out.print("   " +
			     ((DeInteger)(elementAt(j-1))).getValue() +
			     " : ");
	    j += 2;
	}
    }


  /**
   * display the linear combinator, v, using indexes instead of imus 
   * to standard output.
   * @param v a linear combinator
   * @param i width in characters of indentation
   */
  
  
    public void presentBrief(Vector v, int i) {
	int s  = v.size();
	int j = 1;
	while (j < s) {
	    System.out.print("   " +
			     ((DeDouble)(v.elementAt(j))).getValue() + 
			     " : ");
	    System.out.print("   " +
			     ((DeInteger)(v.elementAt(j-1))).getValue() +
			     " : ");
	    j += 2;
	}
    }
}
