package gov.nist.nlpir.irf.query;

import gov.nist.nlpir.irf.matcher.Matcherbp;
import java.util.Vector; 

/** 
 *
 * Holds Query Modality information to be used in retrieval.
 *
 * @version $Revision: 1.2 $
 *
 * @author
 * This software was produced by NIST, an agency of the U.S. government,  
 * and by statute is not subject to copyright in the United States.        
 * Recipients of this software assume all responsibilities associated      
 * with its operation, modification and maintenance.  
 */

public class QueryModalityUnit {

    /**
     * The name of the index associated with this query modality unit.
     */
   private String indexName;

    /** 
     * The Matcher object associated with this query modality unit.
     */
    private Matcherbp matcher;

    /** 
     * The combination of indexing weights and matching degree.
     */
    private IdxWeightsWithMatchDegree combiner;

    /**
     * The document type.
     */
    private String docType;

    /** 
     * The name of the attribute associated with this query modality.
     */
    private String attributeName;

    /** 
     * The matching threshold for this query modality.
     */
    private double thresholdMD;

    /**
     * The weighting function for this query modality
     */
    private double weight;

    /** Serves for accessing the appropriate collection of
     * indexing features from the ordered collection of collections
     * of indexing features derived for the query document. 
     */
    private int number;
			   
			   
    private Vector conditions;
    
    /** 
     * Creates a QueryModalityUnit and initializes IndexName, DocType, AttributeName and
     * ThresholdMD. The Weight is set to 1.0 by default.
     *
     * @param idx Index name.
     * @param ma Matcher
     * @param co Combiner
     * @param doc Document type.
     * @param attr Attribute name.
     * @param threshold ThresholdMD
     */

    public QueryModalityUnit(String idx, Matcherbp ma, IdxWeightsWithMatchDegree co, String doc, 
		   String attr, double threshold) {
	combiner = co;
	matcher = ma;
	indexName = idx;
	docType = doc;
	attributeName = attr;
	thresholdMD = threshold;	
	weight = 1.0; // default
    }

    /** 
     * Sets the Conditions of the QueryModalityUnit to c.
     *
     * @param c Conditions
     */

    public void setConditions(Vector c){
	conditions = c;
    }
    
    /** 
     * Gets the Conditions of the QueryModalityUnit.
     *
     * @return Conditions
     */

    public Vector getConditions(){
	return conditions;
    }
    
    /** 
     * Gets the IndexName of the QueryModalityUnit.
     *
     * @return Index name
     */

    public String getIndexName() {
	return indexName;
    }
    
    /** 
     * Gets the Matcher of the QueryModalityUnit.
     *
     * @return Matcher
     */

    public Matcherbp getMatcher() {
	return matcher;
    }
    
    /** 
     * Gets the Combiner of the QueryModalityUnit.
     *
     * @return Combiner
     */

    public IdxWeightsWithMatchDegree getCombiner() {
	return combiner;
    }
    
    /** 
     * Gets the DocType of the QueryModalityUnit.
     *
     * @return Document type
     */

    public String getDocType() {
	return docType;
    }
    
    /** 
     * Gets the AttributeName of the QueryModalityUnit.
     *
     * @return Attribute name
     */

    public String getAttributeName() {
	return attributeName;
    }
    
    /** 
     * Gets the ThresholdMD of the QueryModalityUnit.
     *
     * @return Threshold
     */

    public double getThresholdMD() {
	return thresholdMD;
    }
    
    /** 
     * Gets the Weight of the QueryModalityUnit.
     *
     * @return Weight
     */

    public double getWeight() {
	return weight;
    }
    
    /** 
     * Sets the Weight of the QueryModalityUnit to c.
     *
     * @param w Weight
     */

    public void setWeight(double w) {
	weight = w;
    }
    
    /** 
     * Gets the number of the QueryModalityUnit.
     *
     * @return number
     */

    public int getNumber() {
	return number;
    }
    
    /** 
     * Sets the number of the QueryModalityUnit to c.
     *
     * @param n number
     */

    public void setNumber(int n) {
	number = n;
    }

    public String toString() {
	return("QMU: idx name=" + indexName + ", matcher=" + matcher + ", combiner=" +
	       combiner + ", docType=" + docType + ", attrName=" + attributeName +
	       ", threshold=" + thresholdMD + ", weight=" + weight + ", number=" +
	       number + ", conditions=" + conditions + " end QMU ///");
    }    
}
