package gov.nist.nlpir.irf.result;

import gov.nist.nlpir.irf.feature.ProxyIndexingFeature;

/**
 * Represents a the lowest level result of a search - a single indexing
 * feature (from within a single field within a single document within 
 * a single index)
 *  
 * @version $Revision: 1.2 $ 	
 * @author
 * This software was produced by NIST, an agency of the U.S. government,  
 * and by statute is not subject to copyright in the United States.        
 * Recipients of this software assume all responsibilities associated      
 * with its operation, modification and maintenance.  
 * @see     gov.nist.nlpir.irf.feature.IndexingFeature
 */
public class BasicRetrievalResult {
  

  /** The degree to which the indexing feature in this result matches the
   *   corresponding feature in the query.
   *   Not used. */
  private double matchingDegree;

  /** The score held in the index for this feature. */
  private double score;

  /** The indexing feature which is the basic result of the search. */
  private ProxyIndexingFeature IF;
  


  /**
   * Creates a BasicRetrievalResult with the score and the degree given.
   * @param IF the Indexing Feature concerned
   * @param matchingDegree the matching degree
   * @param score the score of the represented matching
   */
  public  BasicRetrievalResult(ProxyIndexingFeature IF, double matchingDegree,
			       double score) {
    this.IF = IF;
    this.matchingDegree = matchingDegree;
    this.score = score;
  }
  


  /**
   * Creates a BasicRetrievalResult, but assumes the score and the matching
   * degree are 0.
   * @param IF the Indexing Feature concerned
   */
  public BasicRetrievalResult(ProxyIndexingFeature IF) {
    this.IF = IF;
    matchingDegree = 0.0;
    score = 0.0;
  }
  


  /**
   * Retrieves the Indexing Feature.
   * @return Indexing Feature
   */
  public ProxyIndexingFeature getIndexingFeature() {
    return IF;
  }
  


  /**
   * Gets the matching degree.
   * @return matching degree
   */
  public double getMatchingDegree() {
    return matchingDegree;
  }
  


  /**
   * Gets the score.
   * @returns score
   */
  public double getScore() {
    return score;
  }
  


  /**
   * Sets the matching degree.
   * @param md matching degree
   */
  public void setMatchingDegree(double matchingDegree) {
    this.matchingDegree = matchingDegree;
  }
  


  /**
   * Sets the score.
   * @param score the value of the score
   */
  public void setScore(double score) {
    this.score = score;
  }
  


  /**
   * Returns string representation of BasicRetrievalResult including
   * indexing feature, matching degree, and score.
   * @return string representation of BasicRetrievalResult.
   */
  public String toString() {
    return "BasicRetrievalResult: {IF: " + IF + 
      ", matchingDegree: " + matchingDegree +
      ", score: " + score + "}";
  }
}
