package gov.nist.nlpir.irf.result;

import gov.nist.nlpir.irf.document.ProxyDocument;
import gov.nist.nlpir.irf.util.Support;
import java.util.Collections;
import java.util.Vector;

/** 
 * The result of a search modelling the match between a query and a
 * document collection. <br><br>
 * This is the highest level of search result. Each element summarizes
 * the results for a document. It is sortable but not sorted at
 * creation.
 * 
 * @version $Revision: 1.1 $
 * 
 * @author
 * This software was produced by NIST, an agency of the U.S. government,  
 * and by statute is not subject to copyright in the United States.        
 * Recipients of this software assume all responsibilities associated      
 * with its operation, modification and maintenance.  
 */

public class RetrievalResult {


  /** Contains the set of results for individual documents 
   * (ResultForDocMatchingQuery) which match the query. 
   */
  private Vector resultList;
    
  /** 
   * Creates a RetrievalResult for the supplied results.
   * @param results contains ResultForDocMatchingQuery elements.
   */
  public RetrievalResult(Vector results) {
    resultList = results;
  }
    


  /**
   * Sorts the result based on criteria for comparing 
   * ResultForDocMatchingQuery objects
   */
  public void sort() {
    Collections.sort(resultList, new ResultForDocMatchingQuery());
  }
    


  /**
   * Presents the elements of the list of results.
   * @param  interactive The default value FALSE means
   * that it will not be interactive. Interactive means that the user
   * will be asked if they want to see more elements.
   */
  public void present(boolean interactive) {
  }



  /**
   * Returns the number-th document in the list.
   * @param  number index of desired document.
   * @return The requested Document or null if number is an invalid list
   * index.
   */
  public ProxyDocument getDocNo(int number) {
    // Note: number has range [1-size], list internals has range [0-(size-1)]
    if (number > resultList.size())
      Support.errorMessage("RetrievalResult::getDocNo()" + 
			   "Document no does not exist");
    else 
      return ((ResultForDocMatchingQuery)(resultList.elementAt(number-1))).
	getDocument();

    return null;
  }
    


  /**
   * Returns the i-th element of the list.
   * @param  i index of desired document.
   * @return The requested element.
   */
  public ResultForDocMatchingQuery getListElementNo(int i) {
    return ((ResultForDocMatchingQuery)(resultList.elementAt(i-1)));
  }



  /**
   *   Returns number of documents in the result
   *   @return number of documents in the result
   */
  public int getSize() { return resultList.size(); }



  /**
   *   Returns the whole list of results as a vector.
   *   @return The whole result list.
   */
  public Vector getResultVector() { return resultList; }
    


  /**
   *   Makes the result list equal to the argument.
   *   @param  c The new list.
   */
  public void setResultVector(Vector c) { resultList = c; }

}
