package gov.nist.nlpir.irfapps;

import gov.nist.nlpir.irfapps.trec.fbis.FbisConverter;
import gov.nist.nlpir.irfapps.hci.Bib2AppDocConv;

import gov.nist.nlpir.irf.query.Combinator;
import gov.nist.nlpir.irf.query.LinearCombinator;
import gov.nist.nlpir.irf.query.StructuredCombinator;

import gov.nist.nlpir.irf.modality.IndexingModalityUnit;
import gov.nist.nlpir.irf.modality.IndexingModalities;
import gov.nist.nlpir.irf.modality.RetrievalModalities;
import gov.nist.nlpir.irf.document.Document;
import gov.nist.nlpir.irf.document.ProxyDocument;
import gov.nist.nlpir.irf.document.DocCollection;

import gov.nist.nlpir.irf.index.ProxyIndex;
import gov.nist.nlpir.irf.feature.ProxyIndexingFeature;
import gov.nist.nlpir.irf.feature.ProxyIndexingFeature;
import gov.nist.nlpir.irf.feature.ProxyIdxFeatWordInString;
import gov.nist.nlpir.irf.feature.ProxyIdxFeatWordInText;
import gov.nist.nlpir.irf.result.ResultForDocMatchingQuery;
import gov.nist.nlpir.irf.result.RetrievalResult;

import gov.nist.nlpir.irf.irfman.IrfGlobal;
import gov.nist.nlpir.irf.irfman.IrfManager;

import gov.nist.nlpir.irf.de.DataElem;
import gov.nist.nlpir.irf.de.ProxyDataElem;
import gov.nist.nlpir.irf.de.DeDouble;
import gov.nist.nlpir.irf.de.ProxyDeDouble;
import gov.nist.nlpir.irf.de.DeHtml;
import gov.nist.nlpir.irf.de.ProxyDeHtml;
import gov.nist.nlpir.irf.de.DeInteger;
import gov.nist.nlpir.irf.de.ProxyDeInteger;
import gov.nist.nlpir.irf.de.DeString;
import gov.nist.nlpir.irf.de.ProxyDeString;
import gov.nist.nlpir.irf.util.Support;
import gov.nist.nlpir.irf.conversion.IrfConverter;
import gov.nist.nlpir.irf.proxy.VirtualProxy;

import java.lang.reflect.Field;
import java.lang.reflect.Constructor;
import java.util.Vector;
import java.util.Enumeration;
import java.io.FileWriter;
import java.io.IOException;
import java.io.FileNotFoundException;
import java.text.DecimalFormat;

/**
 * A sample application of text-based retrieval which enables the indexing and searching
 * of a document collection through a simple, tty-style menu-driven interface.
 *
 * @version $Revision: 1.14 $ 
 *
 * @author
 * This software was produced by NIST, an agency of the
 * U.S. government, and by statute is not subject to copyright in the
 * United States.  Recipients of this software assume all
 * responsibilities associated with its operation, modification and
 * maintenance.
 */


public class SampleApp {
  
  /** The file/directory name separator (e.g. "/" for Unix, "\" for DOS) 
   */
  private static String slash =  new String(System.getProperty("file.separator"));


  /** Flag to indicate whether to create indexes by scanning each document once 
   * for each index or by scanning each document once only and incrementing each 
   * index for the document scanned */
  private static boolean populateByIndexThenDoc = false;


  /** Number of processsing stage at which to stop */
  private static int finalStage = 0;


  /** Flag indicating whether this invocation should perform only retrieve 
   * operations*/
  private static boolean retrieveOnly = false;


  /** In batch mode, the document number from which to begin indexing */
  private static int fromDoc = -1;


  /** In batch mode, the document at which to end indexing */
  private static int toDoc = -1;


  /** The name of the file containing the document collection to be indexed in 
   * its "raw" form*/
  private static String fileName = "."+slash+"hci"+slash+"data"+slash+"bib1000";

  //slash+"home"+slash+"irfColls"+slash+"fbis.25"+slash+"text.nsgmls";


  /** The name of a file containing a query to be used to search the collection */
  private static String queryName = "."+slash+"hci"+slash+"data"+slash+"q1";

  //slash+"home"+slash+"irfColls"+slash+"fbis.25"+slash+"query.nsgmls";


 /** The name of the class to be used in converting raw document data to document
  * objects */
  private static String converterName = "gov.nist.nlpir.irfapps.hci.Bib2AppDocConv";


  /** The name of the directory from which the Irf Manager is to be read or 
   * to which it is to be saved */
  private static String irmDir = null;


  /** The name of the directory from which the index files (DB*) are to be read or 
   *  to which they should be writen */
  private static String indexDir = null;


  /** The converter class to be used in converting documents from the "raw" to 
   * an indexable form */
  private static IrfConverter theConverter = null;


  /** The document collection currently in use */
  private static DocCollection currentDocCollection = null;
    

  /**
   * The main method of the application, handles command line options.
   * @param argv argument vector  
   */
  public static void main(String argv[]) {

    for (int i = 0; i < argv.length; i++) {  
      if (argv[i].compareTo("help") == 0) {
	System.out.print("available commands:\n" +
			 "  help      -- you're getting it now.\n" +
			 "  irmDir P  -- use path P to find/save the serialized IrfManager.\n" +
			 "  indexDir P  -- use path P to find/save the index data for a new collection.\n" +
			 "  noinput   -- turn off user input.\n" +
			 "    index   -- noinput + indexing only in noinput mode.\n" +
			 "    update  -- noinput + indexing and updating only in noinput mode.\n" +
			 "    retrieve-- noinput + retrieval from first existing collection only in noinput mode.\n" +
			 "    from X  -- first document to index. Default is 1.\n" +
			 "    to Y    -- last document to index. Default is 20.\n" +
			 "    with Z  -- data file to use for the index. Default is " + fileName + ".\n" +
			 "    query Q -- data file to use for the query. Default is " + queryName + ".\n" +
			 "    converter C -- the name of the document converter class. Default is " + converterName + ".\n" +
			 "    byindex -- Add indexing features to one index from all docs for\n" +
			 "              that index before going on to the next index. Default is\n" +
			 "              false: add indexing features from a doc to all relevant\n" +
			 "              indexes before going on the next doc.\n"+     
			 "  detail    -- turn detailed tracing.\n" +
			 "  medium    -- turn medium (e.g., GC) tracing.\n" +
			 "  coarse    -- turn coarse tracing.\n");
	System.exit(0);
      } else if (argv[i].compareTo("noinput") == 0) {
	System.out.println("Support.setPromptUser(false);");
	Support.setPromptUser(false);
      } else if (argv[i].compareTo("index") == 0) {
	if (Support.getPromptUser()) {
	  Support.errorMessage("flag index: you must first specify noinput.");
	  System.exit(-1);
	} else if (finalStage == 0)
	  finalStage = 6;
	else {
	  Support.errorMessage("Hey, only one of noinput additionnal flags can be set");
	  System.exit(-1);
	}
      } else if (argv[i].compareTo("update") == 0) {
	if (Support.getPromptUser()) {
	  Support.errorMessage("flag update: you must first specify noinput.");
	  System.exit(-1);
	} else if (finalStage == 0)
	  finalStage = 7;
	else {
	  Support.errorMessage("Hey, only one of noinput additional flags can be set");
	  System.exit(-1);
	}

      } else if (argv[i].compareTo("retrieve") == 0) 
	{
	if (Support.getPromptUser()) 
	  {
	  Support.errorMessage("flag update: you must first specify noinput.");
	  System.exit(-1);
	  } 
	else 
	  retrieveOnly = true;

      } else if (argv[i].compareTo("from") == 0) {
	if (Support.getPromptUser()) {
	  Support.errorMessage("flag from: you must first specify noinput.");
	  System.exit(-1);
	}
	try {
	  fromDoc = Integer.parseInt(argv[i + 1]);
	  i++;
	} catch (NumberFormatException nfe) {
	  Support.errorMessage("You need an integer after the from parameter.");
	} catch (ArrayIndexOutOfBoundsException ioobe) {
	  Support.errorMessage("You need an integer after the from parameter.");
	}
      } else if (argv[i].compareTo("to") == 0) {
	if (Support.getPromptUser()) {
	  Support.errorMessage("flag to: you must first specify noinput.");
	  System.exit(-1);
	}
	try {
	  toDoc = Integer.parseInt(argv[i + 1]);
	  i++;
	} catch (NumberFormatException nfe) {
	  Support.errorMessage("You need an integer after the to parameter.");
	} catch (ArrayIndexOutOfBoundsException ioobe) {
	  Support.errorMessage("You need an integer after the to parameter.");
	}
      } else if (argv[i].compareTo("with") == 0) {
	if (Support.getPromptUser()) {
	  Support.errorMessage("flag with: you must first specify noinput.");
	  System.exit(-1);
	}
	try {
	  fileName = argv[i + 1];
	}
	catch (ArrayIndexOutOfBoundsException ioobe) {
	  Support.errorMessage("You need a string after the with parameter.");
	  System.exit(-1);
	}
	i++;
      } else if (argv[i].compareTo("query") == 0) {
	if (Support.getPromptUser()) {
	  Support.errorMessage("flag query: you must first specify noinput.");
	  System.exit(-1);
	}
	try {
	  queryName = argv[i + 1];
	}
	catch (ArrayIndexOutOfBoundsException ioobe) {
	  Support.errorMessage("You need a string after the query parameter.");
	  System.exit(-1);
	}
	i++;
      } else if (argv[i].compareTo("converter") == 0) {
	if (Support.getPromptUser()) {
	  Support.errorMessage("flag query: you must first specify noinput.");
	  System.exit(-1);
	}
	try {
	  converterName = argv[i + 1];
	}
	catch (ArrayIndexOutOfBoundsException ioobe) {
	  Support.errorMessage("You need a string after the converter parameter.");
	  System.exit(-1);
	}
	i++;
      } else if (argv[i].compareTo("irmDir") == 0) {
	try {
	  irmDir = argv[i + 1];
	}
	catch (ArrayIndexOutOfBoundsException ioobe) {
	  Support.errorMessage("You need a string after the irmDir parameter.");
	  System.exit(-1);
	}
	i++;
      } else if (argv[i].compareTo("indexDir") == 0) {
	try {
	  indexDir = argv[i + 1];
	}
	catch (ArrayIndexOutOfBoundsException ioobe) {
	  Support.errorMessage("You need a string after the indexDir parameter.");
	  System.exit(-1);
	}
	i++;
      } else if (argv[i].compareTo("byindex") == 0) {
	if (Support.getPromptUser()) {
	  Support.errorMessage("flag with: you must first specify noinput.");
	  System.exit(-1);
	}
	populateByIndexThenDoc = true;
      } else if (argv[i].compareTo("detail") == 0) {
	System.out.println("Support.setTraceLevel(Support.DETAIL_TRACE);");
	Support.setTraceLevel(Support.DETAIL_TRACE);
      } else if (argv[i].compareTo("medium") == 0) {
	System.out.println("Support.setTraceLevel(Support.MEDIUM_TRACE);");
	Support.setTraceLevel(Support.MEDIUM_TRACE);
      } else if (argv[i].compareTo("coarse") == 0) {
	System.out.println("Support.setTraceLevel(Support.COARSE_TRACE);");
	Support.setTraceLevel(Support.COARSE_TRACE);
      } else
	System.out.println(argv[i] + ": parameter not recognized.");
    }	
    ApplicationMain(irmDir,indexDir);
    System.exit(0);
  }
  
  /**
   * Controls the execution of the application through presentation of menus
   * and processing of input. In batch(noinput) mode,
   * user input is disabled and the application uses a preprogrammed 
   * execution sequence.
   * @param pathToSerializedIrfManagerFiles The directory in which the 
   * serialized Irf Manager was stored.
   * @param pathToIndex The directory in which the index was stored.
   */			
  
  static void ApplicationMain(String pathToSerializedIrfManagerFiles,
			      String pathToIndex) {
    int stage = -1;
    long timestamp = 0;


    /*--------------------------------------------------------------------
     * If a serialized IrfManager exists, deserialize and use it;
     * Otherwise, create a new one.
     *-------------------------------------------------------------------*/
    try {
      IrfGlobal.gIrfManager = new IrfManager(pathToSerializedIrfManagerFiles,true);
    } catch (FileNotFoundException e) {
      Support.errorMessage("IrfManager: Unexpected error in constructing");
      System.exit (-1);
    }

    if (Support.MEDIUM_TRACE <= Support.traceLevel)
      if (VirtualProxy.getInMemoryProxiesByOidSize() <= 100)
	VirtualProxy.presentInMemoryProxies();
      else
	Support.errorMessage("More than 100 proxies, I don't print them.");
    
    String response = "1";
    int choice = 1;
    while (choice != 0) {
      Support.userPrompt("\nMENU:\n"
			 + " 1 - Display available collections\n"
			 + " 2 - Choose a collection to work with\n"
			 + " 3 - Present all indexes for current collection\n"
			 + " 4 - Display index statistics for current collection\n"
			 + " 5 - Display stopword statistics for current collection\n"
			 + " 6 - Add document(s) to the current collection\n"
			 + " 7 - Update current collection's indexes (idf etc)\n"
			 + " 8 - Retrieve \n"
			 + " 9 - Set indexing modalities for current collection\n"
			 + "11 - Show IR manager\n"
			 + "12 - Show statistics on all indexes for current collection\n"
			 + "13 - Dump in-memory proxy table\n"
			 + "\n"
			 + " 0 - QUIT\n"
			 + "Choice : " );
      if (Support.getPromptUser())
	try {
	  response = Support.readLine();
	  if (response.length() > 0)
	    choice = Integer.parseInt(response);
	  else
	    choice = 1;
	} catch (NumberFormatException error) {
	  Support.errorMessage("illegal input: " + error);
	  choice = -1;
	}
      else { // no prompt user
	if (stage == finalStage) stage = 0;
	else if (stage == -1) stage = 2;
	else if (stage == 2) 
	  {
	  if (retrieveOnly)
	    stage = 8;
	  else
	    stage = 9;
	  }
	else if (stage == 9) stage = 6;
	else if (stage == 6) stage = 7;
	else if (stage == 7) stage = 4;
	else if (stage == 4) stage = 8;
	else if (stage == 8) stage = 0;
	choice = stage;
      }
      if (1 <= Support.traceLevel)
	  Support.traceMessage("ApplicationMain choice: " +
			       Integer.toString(choice));
      switch (choice) {
      case 0: {
	System.out.println("Shutting down the Irf Manager...");
	timestamp = Support.getTimeInMillis();

	// For now save all document collections - later ask user

	Vector v = IrfGlobal.gIrfManager.getModifiedDocColls();

	if (Support.COARSE_TRACE <= Support.traceLevel)
	  Support.traceMessage("Saving " + v.size()+ " DocCollections...");

	for (int i = 0; i < v.size(); i++) {
	  if (Support.COARSE_TRACE <= Support.traceLevel)
	    Support.traceMessage("Saving DocCollection: "+
				 ((DocCollection)v.elementAt(i)).getName());
	  ((DocCollection)v.elementAt(i)).save();
	}

	if (Support.COARSE_TRACE <= Support.traceLevel)
	    Support.traceMessage("Elapsed seconds to save " + v.size() +
				 " collection(s) = " + 
				 (Support.getTimeInMillis()-timestamp)/1000);

	timestamp = Support.getTimeInMillis();
	IrfGlobal.gIrfManager.shutdown();
	if (Support.COARSE_TRACE <= Support.traceLevel)
	  Support.traceMessage("Elapsed seconds to shutdown IrfManager = " +
			       (Support.getTimeInMillis()-timestamp)/1000);

      } break;
      
      case 1: {
	System.out.println("Presenting collection(s)...");
	Enumeration it = 
	  IrfGlobal.gIrfManager.getInfoServer().getDocCollIter();
	int i = 0;
	while (it.hasMoreElements()) {
	  i++;
	  DocCollection aDocColl = (DocCollection)it.nextElement();
	  System.out.println( i + "\tName: " + aDocColl.getName() + 
			    "\n\tDesc: " + aDocColl.getDescription() + 
			    "\n\t Loc: " + aDocColl.getIndexLocation()); 
	}
      } break;
      
      case 2: {
	if (currentDocCollection != null && 
	    currentDocCollection.getModified()) {
	  System.out.println("Unsaved modifications to collection will be lost...");
	}

	currentDocCollection = getCurrentDocCollection(indexDir);

	// Notify the Infoserver where the brokers can find the indexes
	IrfGlobal.gIrfManager.getInfoServer().
	  setLocationOfBrokeredStorage(currentDocCollection.getIndexLocation());

      } break;
      
      case 3: {
	if (currentDocCollection == null)
	  Support.errorMessage("Please choose a collection to work with");
	else 
	  {
	  System.out.println("Presenting indexes for current collection...");
	  Enumeration it = currentDocCollection.getIndexIter();
	  while (it.hasMoreElements())  {
	    ProxyIndex anIndex = (ProxyIndex)it.nextElement();
	    if (anIndex == null)
	      Support.errorMessage("ApplicationMain(): NULL index! (1)");
	    System.out.print("Presenting " + anIndex.getName() + "...\n" ); 
	    anIndex.present();
	    anIndex.makeLightweight();
	  }
	}
      } break;
      
      case 4: {
	if (currentDocCollection == null)
	  Support.errorMessage("Please choose a collection to work with");
	else {
	  System.out.println("Presenting index statistics for current collection...");
	  Enumeration it = currentDocCollection.getIndexIter();
	  while (it.hasMoreElements())  {
	    ProxyIndex anIndex = (ProxyIndex)it.nextElement();
	    System.out.println("Presenting statistics for " + 
			       anIndex.getName() + "..." );
	    anIndex.presentStatistics();
	    anIndex.makeLightweight();
	  }  
	}
      } break;
      
      case 5: {
	if (IrfGlobal.gTextStatistics != null)
	  IrfGlobal.gTextStatistics.present();
	else  
	  Support.errorMessage("Sorry, no statistics available");
      } break;
      
      case 6: {
	if (currentDocCollection == null)
	  Support.errorMessage("Please choose a collection to work with");   
	else if (currentDocCollection.getIndexingModalities() == null)
	  Support.errorMessage("Please set the indexing modalities for the current collection");
	else {
	  populateColl(currentDocCollection);
	  currentDocCollection.save();

	  if (Support.MEDIUM_TRACE <= Support.traceLevel)
	    if (VirtualProxy.getInMemoryProxiesByOidSize() <= 100)
	      VirtualProxy.presentInMemoryProxies();
	    else
	      Support.errorMessage("More than 100 proxies, I don't print them.");
	}
      } break;
      
      case 7: {
	if (currentDocCollection == null)
	  Support.errorMessage("Please choose a collection to work with");   
	else if (currentDocCollection.getNumDocs() > 0) {
            System.out.println("Updating indexes...");
	    Enumeration it = currentDocCollection.getIndexIter();
	    timestamp = Support.getTimeInMillis();
	  
	    IrfGlobal.gIrfManager.getInfoServer().setMaxHandlesByOidCacheEntries(1);

	    while (it.hasMoreElements()) {
		ProxyIndex anIndex = (ProxyIndex)it.nextElement();
		if (anIndex == null)
		    Support.errorMessage("ApplicationMain(): NULL index!");
		if (Support.COARSE_TRACE <= Support.traceLevel)
		    Support.traceMessage("Updating " + anIndex.getName());
		anIndex.update();
		anIndex.makePersistent();
		anIndex.replaceRealObject();
		anIndex.makeLightweight();
		
	    }
	  
	    if (Support.COARSE_TRACE <= Support.traceLevel)
		Support.traceMessage("Elapsed seconds to update = " +
			       (Support.getTimeInMillis() - timestamp) / 1000);
	  
	  if (Support.MEDIUM_TRACE <= Support.traceLevel)
	    if (VirtualProxy.getInMemoryProxiesByOidSize() <= 100)
	      VirtualProxy.presentInMemoryProxies();
	    else
	      Support.errorMessage("More than 100 proxies, I don't print them.");
	} else
	  Support.errorMessage("ApplicationMain(): NULL index!");
	
      } break;
      
      case 8: {
	if (currentDocCollection == null)
	  Support.errorMessage("Please choose a collection to work with");   
	else if (currentDocCollection.getIndexingModalities() == null)
	  Support.errorMessage("Please set the indexing modalities for the current collection");
	else {
	  // For now, let retrieval modalities be same as indexing ones
	  RetrievalModalities retrievalModalities = 
	    new RetrievalModalities(currentDocCollection.getIndexingModalities());
	  retrieve(retrievalModalities, currentDocCollection.getIndexingModalities());
	  if (Support.MEDIUM_TRACE <= Support.traceLevel)
	    if (VirtualProxy.getInMemoryProxiesByOidSize() <= 100)
	      VirtualProxy.presentInMemoryProxies();
	    else
	      Support.errorMessage("More than 100 proxies, I don't print them.");
	}

      } break;
      
      case 9: {
	if (currentDocCollection == null)
	  Support.errorMessage("Please choose a collection to work with");   
	else if (currentDocCollection.getIndexingModalities() == null ||
		 currentDocCollection.getNumDocs() == 0)
	  setIndexingModalities(currentDocCollection); 
	else
	  Support.errorMessage("Indexing modalities cannot be changed for the current collection: " +
			       currentDocCollection.getName());  
	if (Support.MEDIUM_TRACE <= Support.traceLevel)
	  if (VirtualProxy.getInMemoryProxiesByOidSize() <= 100)
	    VirtualProxy.presentInMemoryProxies();
	  else
	    Support.errorMessage("More than 100 proxies, I don't print them.");
      } break;
      
      
      case 11: {
	IrfGlobal.gIrfManager.present();
      } break;
      
      case 12: {
	if (currentDocCollection == null)
	  Support.errorMessage("Please choose a collection to work with");
	else {
	  int depth = 1;
	  System.out.print("In each index, for each table:\n" +
			   "depth 1 prints the size of the contained vectors,\n" +
			   "depth 2 gives for each vector its key and the number of its elements,\n" +
			   "depth 3 gives the key of each vector and prints its content truncated to X characters,\n" +
			   "depth 4 is as 2, only the size of the vectors are printed one behind another.\n" +
			   "Depth of the statistics (1-4): ");
	  String resp = "1";
	  try {
	    resp = Support.readLine();
	    if (resp.length() > 0)
	      depth = Integer.parseInt(resp);
	  } catch (NumberFormatException error) {
	    Support.errorMessage("illegal input: " + error);
	    depth = 1;
          }
	  int maxLength = 10;
	  if (depth == 3) {
	    System.out.print("X: Number of characters (" + maxLength + "): ");
	    resp = "1";
	    try {
	      resp = Support.readLine();
	      if (resp.length() > 0)
	        maxLength = Integer.parseInt(resp);
	    } catch (NumberFormatException error) {
	      Support.errorMessage("illegal input: " + error);
	    }
	  }
	  Enumeration it = currentDocCollection.getIndexIter();
	  while (it.hasMoreElements()) {
	    ProxyIndex anIndex = (ProxyIndex)it.nextElement();
	    if (anIndex == null)
	      Support.errorMessage("ApplicationMain(): NULL index! (1)");
	    else {
	      System.out.println("\nINDEX " + anIndex.getName());
	      anIndex.showStatistics(depth, maxLength);
	      anIndex.makeLightweight();
	    }
	  }
	}
      } break;
      
      case 13: {
	VirtualProxy.presentInMemoryProxies();
      } break;
      
      default : {
	Support.errorMessage("Sorry, you entered an invalid menu option. Try again, please.");
      } break;
      }
    } // while choice
  }
  
  static ProxyDocument dummy;
  
  /**
   * SetGet the current document collection.
   * @param pathToIndex The directory in which the index was saved.
   * @return A vector representing an existing or new document collection.
   */
  static DocCollection getCurrentDocCollection(String pathToIndex) {
    
    /*-------------------------------------------------------------------
     * Let the user choose from a list of current document collections
     * or start a new collection
     *-------------------------------------------------------------------*/
    Support.userPrompt("\nSelect a document collection by number if any available:\n--------------------\n");
    
    Enumeration it= IrfGlobal.gIrfManager.getInfoServer().getDocCollIter();
    Vector collVector = new Vector();
    int numColls = 0;
    while (it.hasMoreElements()) {
      numColls++;
      DocCollection aDocColl = (DocCollection)it.nextElement();
      collVector.addElement(aDocColl);
      Support.userPrompt(numColls + "   " + aDocColl.getName());
    }
    Support.userPrompt("\n--------------------\n" +
		       "Or enter 0 to create a new collection: ");
    
    int collSelectionNumber = 0;
    if (Support.getPromptUser())
      do
	try {
	  collSelectionNumber = Integer.parseInt(Support.readLine());
	} catch (NumberFormatException error) {
	  Support.errorMessage("illegal input: " + error);
	  collSelectionNumber = -1;
	  Support.userPrompt("Nr of Collection (0 for a new one): ");
	}
      while (collSelectionNumber == -1);
    else  // no user prompt
      if (numColls == 0)           // no collections exist; create one
	collSelectionNumber = 0;
      else                         // at least one collection exists; use first
	collSelectionNumber = 1;
    
    DocCollection curColl = null;
    
    if (collSelectionNumber == 0) {
      
      // If in interactive mode, ask for name, description, and/or location
      if (Support.getPromptUser()) {
	// ------ Ask for name ----------
	String name = "";
	while (name.compareTo("") == 0) { 
	  Support.userPrompt("Enter a name for the new collection or . to skip: ");
	  name = Support.readLine(); 
	}
	if ((name.trim()).compareTo(".") == 0) 
	  name = "No name";
	
	// ------ Ask for description ---
	String desc = "";
	while (desc.compareTo("") == 0) { 
	  Support.userPrompt("Enter a description for the new collection or . to skip: ");
	  desc = Support.readLine(); 
	}
	if ((desc.trim()).compareTo(".") == 0) 
	  desc = "No description";	      
	
	// ------ Ask for directory -----
	
	String dir = "";
	while (dir.compareTo("") == 0) { 
	  Support.userPrompt("Enter the fully qualified directory to hold the indexes" +
			     " or . to use the current directory: ");
	  dir = Support.readLine(); 
	}
	if ((dir.trim()).compareTo(".") == 0) 
	  dir = null;
	
	curColl = new DocCollection(name, desc, dir);
	
	theConverter = null;
	
	while (theConverter == null) {
	  Support.userPrompt("Enter the full name of the converter class: ");
	  converterName = Support.readLine();
	  try {
	    Class converterClass = Class.forName(converterName);
	    theConverter = (IrfConverter)converterClass.newInstance();
	  } catch(NoClassDefFoundError err) {
	    Support.errorMessage("The specified converter class cannot be found." +
				 "Please Try Again.");
	  } catch (ClassNotFoundException cnfe) {
	    Support.errorMessage("This converter class doesn't seem to exist. Don't forget " +
				 "you need the FULL name, ie package1.package2...." +
				   "Something2OtherThingConverter");
	  } catch (Exception e) {
	    Support.errorMessage("Exception: " + e + "  Try again...");
	  }
	}
      } else {  // use defaults (possibly overridden by parms to SampleApp

	curColl = new DocCollection("No-Name", "No-Description", pathToIndex);
	try 
	  {
	  Class converterClass = Class.forName(converterName);
	  theConverter = (IrfConverter)converterClass.newInstance();
	  } 
	catch(NoClassDefFoundError err) 
	  {
	  Support.errorMessage("The specified converter class cannot be found." +
			       "Please Try Again.");
	  System.exit(-1);
	  } 
	catch (ClassNotFoundException cnfe) 
	  {
	  Support.errorMessage("This converter class doesn't seem to exist. Don't forget " +
			       "you need the FULL name, ie package1.package2...." +
			       "Something2OtherThingConverter");
	  System.exit(-1);
	  } 
	catch (Exception e) 
	  {
	  Support.errorMessage("Exception: " + e + "  Try again...");
	  System.exit(-1);
	  }
      }
      
      curColl.setConverterClassName(converterName);
      
      IrfGlobal.gIrfManager.getInfoServer().addDocColl(curColl);
      curColl.save();
      
    } else if (collSelectionNumber > numColls) {
      Support.userPrompt("\nYour selection (" + collSelectionNumber + ") is invalid  - try again, please\n"); 
      return getCurrentDocCollection(indexDir);
      
    } else {
      curColl = (DocCollection)collVector.elementAt(collSelectionNumber - 1);
      try {
	Class converterClass = Class.forName(curColl.getConverterClassName());
	theConverter = (IrfConverter)converterClass.newInstance();
      } catch (ClassNotFoundException cnfe) {
	Support.errorMessage("This converter class doesn't seem to exist.");
      } catch (IllegalAccessException iae) {
	Support.errorMessage("Error: " + iae + "  Try again...");
      } catch (InstantiationException ie) {
	Support.errorMessage("Error: " + ie + "  Try again...");
      }
      dummy = buildDummyDoc(theConverter);
    }
    if (Support.DETAIL_TRACE <= Support.traceLevel)
      curColl.present();
    return curColl;
  }
  
  
  /**
   * Set the indexing modalities for the current document collection.
   *
   * @param docColl The document collection for which to set the indexing
   * modalities.
   */
  static void setIndexingModalities(DocCollection docColl) {
    
    dummy = buildDummyDoc(theConverter); // build a dummy document
    IndexingModalities indexingModalities = docColl.getIndexingModalities();

    // Allow setting/update as long as no docs added to collection
    if (indexingModalities == null || docColl.getNumDocs() == 0) {
      if (indexingModalities == null) {
	indexingModalities = new IndexingModalities();
	docColl.setIndexingModalities(indexingModalities);
      }

      if (Support.getPromptUser())
	docColl.editIndexingModalities(dummy);
      else 
	docColl.setDefaultIndexingModalities(dummy);

    } else
      Support.errorMessage("Indexing modalities cannot be changed for the current collection: "
			   + docColl.getName());
  }

  /**
   * Build a dummy (empty) proxy Document
   *
   * @param aConverter a IrfConverter for documents of the type to be
   * created.
   * @return a dummy ProxyDocument
   */
  static ProxyDocument buildDummyDoc(IrfConverter aConverter) {
    // Dummy document for default modalities
    ProxyDocument dummy = null;
    try {    
      Class stringClass = "".getClass();
      Class[] argTypes = { stringClass };
      Object[] constructorArgs = { "" };
      
      Class realDummyClass = theConverter.getRealDocClass();
      Document realDummy = (Document)realDummyClass.newInstance();
      
      Field[] fields = realDummyClass.getFields();
      int nrOfFields = fields.length;
      String UID = "serialVersionUID";
      for (int i = 0; i < nrOfFields; i++) {
	Field currentField = fields[i];
	if (currentField.getName().equals(UID))
	  continue;
	Class currentFieldClass = currentField.getType();
	Constructor currentConstruct = currentFieldClass.getConstructor(argTypes);
	ProxyDataElem currentPDE = (ProxyDataElem)currentConstruct.newInstance(constructorArgs);
	currentField.set(realDummy, currentPDE);
      }
      
      Class dummyClass = theConverter.getProxyDocClass();
      Class[] proxyConstructorTypes = { realDummyClass };
      Constructor proxyConstructor = dummyClass.getConstructor(proxyConstructorTypes);
      Object[] proxyConstructorArgs = { realDummy };
      dummy = (ProxyDocument)proxyConstructor.newInstance(proxyConstructorArgs);
    } catch(IllegalAccessException iae) {
      Support.errorMessage("Error in SampleApp.setIndexingModalities: " + iae);
    } catch(InstantiationException ie) {
      Support.errorMessage("Error in SampleApp.setIndexingModalities: " + ie);
    } catch(NoSuchMethodException nsme) {
      Support.errorMessage("Error in SampleApp.setIndexingModalities: " + nsme);
    } catch(java.lang.reflect.InvocationTargetException ite) {
      Support.errorMessage("Error in SampleApp.setIndexingModalities: " + ite);
    }
    return dummy;
  }

  
  /**
   * Create documents, add them to a collection  and index them.
   *
   * @param docColl the document collection to populate
   */ 

  static void populateColl(DocCollection docColl) {
    ProxyDocument rd;
    boolean docOK = true;
    if (Support.getPromptUser()) {
      fileName = "";
      while (fileName == "") {
	Support.userPrompt("Raw data fileName   : ");
	try {
	  fileName = Support.readLine();
	  theConverter.setRawDocLocation(fileName);
	} catch (Exception e) {
	  if (fileName.startsWith(slash))
	    Support.errorMessage("The file (" + fileName + ") can't be found");
	  else
	    Support.errorMessage("The file ("+ System.getProperty("user.dir") + fileName +
				 ") can't be found.\nIs \"" + System.getProperty("user.dir") +
				 "\" the directory you want to be running in?");
	  fileName = "";
	}
      }
    } else
      try {
	theConverter.setRawDocLocation(fileName);
      } catch (Exception e) {
        if (fileName.startsWith(slash))
	  Support.errorMessage("The file (" + fileName + ") can't be found");
	else
	  Support.errorMessage("The file (" + System.getProperty("user.dir") + 
			       slash + fileName +
			       ") can't be found.\nIs \"" + System.getProperty("user.dir")+
			       "\" the directory you want to be running in?");
	System.exit(-1);
      }
    if (!Support.getPromptUser()) {
      if (fromDoc == -1) fromDoc = 1;
      if (toDoc == -1)   toDoc = 20;
    } else {
      fromDoc = -1;
      toDoc   = -1;
    }
    
    while (fromDoc == -1) {
      Support.userPrompt("From which doc : ");
      try {
	fromDoc = Integer.parseInt(Support.readLine());
      } catch (NumberFormatException error) {
	Support.errorMessage("illegal input: " + error);
	fromDoc = -1;
      }
    }
    while (toDoc == -1) {
      Support.userPrompt("To which doc   : ");
      try {
	toDoc = Integer.parseInt(Support.readLine());
      } catch (NumberFormatException error) {
	Support.errorMessage("illegal input: " + error);
	toDoc = -1;
      }
    }
    int nodocs = toDoc - fromDoc + 1;
    int docCount = 1;
    if (Support.DETAIL_TRACE <= Support.traceLevel)
	Support.traceMessage("nodocs: "   + Integer.toString(nodocs)  +
			     " fromDoc: " + Integer.toString(fromDoc) +
			     " toDoc: "   + Integer.toString(toDoc));
    
    Support.userPrompt("Index all selected docs for one index at a time? ");
    if (Support.getPromptUser())
      populateByIndexThenDoc = confirmYN();
    
    Vector newdocs = new Vector(nodocs);
    
    // Ignore first documents:
    while (docCount < fromDoc) {
      try {
	   rd = theConverter.convert1();
	   if (rd == null) {
	       Support.warningMessage("Warning: Beginning document not reached (error in input file?)!");
	       return;
	}
      } catch (IOException error) {
	Support.errorMessage("Error: " + error);
      }
      docCount++;
    }
    
    System.out.println("Converting raw documents...");
    
    if (!populateByIndexThenDoc) 
      System.out.println("Adding indexing features to all appropriate indexes a document at a time...");
    
    long timestamp = Support.getTimeInMillis();
    long lastStamp = timestamp;
    long currentStamp = lastStamp;
    
    // Read rest into db & newdocs
    while (docCount <= toDoc && docOK) {
      if (Support.COARSE_TRACE <= Support.traceLevel)
	  Support.traceMessage("Docs to go: " + nodocs-- +
			 " -------------------------------------------------");
      try {
	rd = theConverter.convert1();
	
	if (rd == null) { // Error condition, e.g. premature eof
	  docOK = false;
	  Support.warningMessage("Warning: End document not reached (error in input file?)!");
	} else {
	  
	  rd.makePersistent();
	  docColl.addDoc(rd);       // adding to persistent db    
	  docCount++;
	  
	  if (!populateByIndexThenDoc) {
	    // For each doc, do all indexes
	    rd.index(docColl.getIndexingModalities(), rd);	       
	    rd.makeLightweight(); // We don't need the doc anymore

	  } else {
	    newdocs.addElement(rd);  // needed for incremental indexing only
	    rd.makeLightweight();    // experimentally in this leg too	    
	  }
	}
      } catch (IOException error) {
	Support.errorMessage("Error: " + error);
      }
      if (nodocs % 100 == 0) { // If you change this number, please change the message below :-)
	currentStamp = Support.getTimeInMillis();
	if (Support.DETAIL_TRACE <= Support.traceLevel)
	    Support.traceMessage("Last 100 documents in " +
				 (currentStamp - lastStamp) / 1000 +
				 " seconds.\n" +
				 VirtualProxy.getInMemoryProxyCounts());
	lastStamp = currentStamp;
      }
    }
    
    nodocs = toDoc - fromDoc + 1;

    if (!populateByIndexThenDoc) {
      /*------------------------------------------------------------------*
       * Done adding features to all the indexes so make them all
       * persistent, update them and make them lightweight.
       *------------------------------------------------------------------*/
      ProxyIndex curIndex = null;
      Vector modalities = (docColl.getIndexingModalities()).getModalities();
      int modalityNumber = modalities.size();
      for (int i = 0; i < modalityNumber; i++) {
	IndexingModalityUnit singleIMU = 
	  (IndexingModalityUnit)modalities.elementAt(i);
	
	curIndex = singleIMU.getIndex(); 
	curIndex.makePersistent();
	curIndex.replaceRealObject();
	curIndex.makeLightweight();
      }
    } else {
      System.out.println("Adding indexing features from " +
			  "all selected documents an index at a time...");
      int nodocs0 = nodocs;
      ProxyIndex curIndex = null;
      Vector modalities = (docColl.getIndexingModalities()).getModalities();
      int modalityNumber = modalities.size();
      for (int i = 0; i < modalityNumber; i++) {
	IndexingModalityUnit singleIMU = (IndexingModalityUnit)modalities.elementAt(i);
	nodocs = nodocs0;
	IndexingModalities singleIM = new IndexingModalities();
	singleIM.addModality(singleIMU);
	if (Support.COARSE_TRACE <= Support.traceLevel)
	    Support.traceMessage("Indexing with modalities: ");
	singleIM.present();
	int docNumber = newdocs.size();
	/*------------------------------------------------------------------*
	 * For each Document/DataElement from which indexing features are to
	 * be extracted and added to the current index.
	 *------------------------------------------------------------------*/ 
	for (int j = 0; j < docNumber; j++) {
	  rd = (ProxyDocument)newdocs.elementAt(j);
	  if (Support.COARSE_TRACE <= Support.traceLevel)
	      Support.traceMessage("Docs to go: " + (nodocs--) +
			  " ------------------------------------------------");
	  rd.index(singleIM, rd);
	  /*----------------------------------------------------------------*
	   * The proxy data element within the current document that is 
	   * associated with this IM has had any indexing features extracted
	   * and won't be revisited during this invocation of populateColl,
	   * so it can be made lightweight. For now we do something simpler
	   * and make the whole proxy document lightweight at the cost of
	   * more I/O.
	   *----------------------------------------------------------------*/
	  rd.makeLightweight();
	}
	
	/*------------------------------------------------------------------*
	 * Done adding features to current index. Be sure it's persistent, 
	 * up-to-date, and then lightweight
	 *------------------------------------------------------------------*/
	curIndex = singleIMU.getIndex(); 
	curIndex.makePersistent();
	curIndex.replaceRealObject();
	curIndex.makeLightweight();
	
      } // end while more IMUs (indexes)
    } // end if incremental indexing (documents within index)
    fromDoc = -1;
    toDoc   = -1;

    if (Support.COARSE_TRACE <= Support.traceLevel)
	Support.traceMessage("Elapsed seconds to convert and index = "+
			     (Support.getTimeInMillis() - timestamp) / 1000);
  }
  
  
  /**
   * Takes as argument a ResultForDocMatchingQuery to be presented 
   * on the screen.
   * The function duplicates the ProxyDocument that the argument refers to, 
   * keeping the original fields that do not need to be changed.
   * DeHtmls and DeStrings, however, may need to be changed to mark
   * query terms that appear in the document. For each of those DEs, a
   * new DeHtml/String object is created, based on the old, but with
   * &lt;em&gt; and &lt;/em&gt; added around each word matching the 
   * query.
   *
   * @param rle  ResultForDocMatchingQuery
   * @return Highlighted version of the Document.
   */
  
  static Document prepareDocPresent(ResultForDocMatchingQuery rle)
    throws ArrayIndexOutOfBoundsException {
    
    // will be of the same class as sourceDoc. Not known at compile time.
    Document result = null; 
    Document sourceDoc = (Document)rle.getDocument().getRealObject();

    // get the value of the different  fields. They should all be ProxyDEs. */
    Vector featureValues = sourceDoc.getFeatureValues(); 

    Class c = sourceDoc.getClass(); 
    Field[] fields = c.getFields();
    int numberOfFields = fields.length;

    try {

      // This way, result and sourceDoc are instances
      // of the same Document subclass. 
      result = (Document)c.newInstance(); 

    } catch (IllegalAccessException iae) {
      System.err.println("SampleApp.prepareDocPresent: " + iae);
    } catch (InstantiationException ie) {
      System.err.println("SampleApp.prepareDocPresent: " + ie);
    }   
 
    // The indexing features that  match the query in sourceDoc
    Vector resultIndexingFeatures = rle.getIndexingFeatures(); 
    int RIF_Number = resultIndexingFeatures.size();

    String UID = "serialVersionUID";
    int valuesIndex = 0;
    for (int i = 0; i < numberOfFields; i++) {
      // will allow the dynamic creation of result thanks to the Reflection API.
      Field currentField = fields[i]; 

      // With Java 2.0 fields[] will contain an entry for serialVersionUID 
      // but featureValues will not! Therefore featureValues has its own 
      // index variable, which is not incremented unless it is used. It is 
      // not used/incremented when we skip the serialVersionUID entry in 
      // fields[] 

      if (currentField.getName().equals(UID))
	continue; // not interested in serialVersionUID field

      ProxyDataElem currentResultField = (ProxyDataElem)featureValues.elementAt(valuesIndex++);

      ProxyDataElem actualResultField = currentResultField; 

      // Fields appear in the same order as features do in the feature Values 
      // Vector, so that this ProxyDE corresponds to currentField.

      // May need to call addHighlight.
      if (currentResultField instanceof ProxyDeString) {
	/* We have to go through every Indexing Feature
	 *  because it's not linked to the field. Mmm.
	 *  Have to check that.                         */
	
	for (int j = 0; j < RIF_Number; j++) { 
	  ProxyIndexingFeature oneResult = 
	    (ProxyIndexingFeature)resultIndexingFeatures.elementAt(j);
	  if (oneResult instanceof ProxyIdxFeatWordInString)
	    try {
	      if (oneResult.getSourceField().getClass().
		  isInstance((DataElem)currentField.get(sourceDoc)))
		if (((DataElem)oneResult.getSourceField()).
		    equals((DataElem)currentField.get(sourceDoc)))
		  if (currentResultField instanceof ProxyDeHtml)
		    actualResultField =
		      new ProxyDeHtml(addHighlight(((ProxyDeHtml)actualResultField).getValue(),
						((ProxyIdxFeatWordInText)oneResult).getWordCount(),
						((ProxyIdxFeatWordInText)oneResult).getParagraphCount()));
		  else // if (currentResultField instanceof ProxyDeString) // already checked.
		    actualResultField =
		      new ProxyDeString(addHighlight(((ProxyDeString)actualResultField).getValue(),
						  ((ProxyIdxFeatWordInString)oneResult).getWordCount(),
						  1));
	    } catch (Exception e) {
	      System.err.println("e: " + e);
	    }
	  } //end for
      } // end if currentResultField is a ProxyDeString
      
      try {
	currentField.set(result, actualResultField);
      } catch (IllegalAccessException iae) {
	System.err.println("SampleApp.prepareDocPresent: " + iae);
      }
    }
    return result;
  }
  
  static String addHighlight(String s, int wantedWord, int wantedParagraph) {
    char[] arraySource = s.toCharArray();
    int pointerSource = 0;
    int pointerResult = 0;
    int lenSource = arraySource.length;
    StringBuffer resultStrBuf = new StringBuffer(lenSource);
    int state = 0;
    int wordCount = 1;
    int paragraphCount = 1;
    
    while (pointerSource < lenSource) {
      char current = arraySource[pointerSource];
      switch (state) {
      case 0: {
	if (current == '<')
	  state = 1;
	else if (Character.isLetterOrDigit(current)) {
	  // Prelude to state 5: is the next word the good one ?
	  if ((wordCount == wantedWord) &&
	      (paragraphCount == wantedParagraph)) {
	    resultStrBuf.append("<em>");
	  }
	  state = 4;
	}
      } break;
      case 1: {
	if (current == 'p')
	  state = 2;
	else
	  state = 3;
      } break;
      case 2: {
	if (current == '>') {
	  state = 0;
	  wordCount = 0;
	  paragraphCount = 0;
	} else
	  state = 3;
      } break;
      case 3: {
	if (current == '>')
	  state = 0;
      } break;
      case 4: {
	if (!Character.isLetterOrDigit(current) ||
	    (pointerSource + 1 == lenSource)) {
	  if ((wordCount == wantedWord) &&
	      (paragraphCount == wantedParagraph)) {
	    resultStrBuf.append("</em>");
	  }
	  wordCount++;
	  if (current == '<')
	    state = 1;
	  else
	    state = 0;
	}
      } break;
      }
      resultStrBuf.append(current);
      pointerSource++;
    }
    return resultStrBuf.toString();
  }
  
  /**
   * Print a specified number of spaces.
   * @param i The number of spaces to print.
   */


    /*
  static void printSpaces(int i) {
    for (int j = 0; j < i; j++)
      System.out.print(' ');
  }
    */
  
  private static final Vector STRUCLEAF(String F, int I1, double W1, int I2, double W2) {
    Vector C = new Vector();
    C.addElement(new DeString(F));
    C.addElement(new DeInteger(I1));
    C.addElement(new DeDouble(W1));
    C.addElement(new DeInteger(I2));
    C.addElement(new DeDouble(W2));
    return C;
  }
  
  private static final void STRUCCOMBRET(Vector C, String F, Object C1, 
					 double W1, int C2, double W2) {
    C.addElement(new DeString(F));
    C.addElement(C1);
    C.addElement(new DeDouble(W1));
    C.addElement(new DeInteger(C2));
    C.addElement(new DeDouble(W2));
  }
  
    /**
     * Request an integer weight to assign to each indexing modality for use in calculating the contribution
     * of the indexing modality to a document's final score/RSV.
     * @param theModalities The set of indexing modalities for this collection.
     * @return An array holding a normalized weight factor for each indexing modality. The normalized 
     * factor is calculated by dividing the integer weight assigned to a given indexing modality by the sum 
     * of the weights assigned to all indexing modalities.
     */
    static double[] requestNormalizedWeights(Vector theModalities){
	int numberOfModalities = theModalities.size();
	Support.userPrompt(
			   "You are going to be prompted for the weight\n" +
			   "of each field in the type of document you're\n" +
			   "using, and you will then be proposed three\n" +
			   "types of combinator (linear, max and min).\n");
	
	int[] weights = new int[numberOfModalities];
	int weightSum = 0;
	for (int i = 0; i < numberOfModalities; ) {
	    String currentFeatureName = ((IndexingModalityUnit)theModalities.elementAt(i)).getFeatureName();
	    if (Support.getPromptUser()) {
		Support.userPrompt("Weight for feature " + currentFeatureName + ": ");
		try {
		    weights[i] = Integer.parseInt(Support.readLine());
		} catch (Exception error) {
		    Support.errorMessage("Couldn't read the weight. Try again.");
		    weights[i] = -1;
		}
	    } else
		weights[i] = 1; // default value
	    
	    if (weights[i] != -1){
		weightSum += weights[i];
		i++;
	    }
	}
	double weightChunk = 1.0 / (double)weightSum;
	double[] normalizedWeights = new double[numberOfModalities];
	for (int i = 0; i < numberOfModalities; i++)
	    normalizedWeights[i] = weightChunk * weights[i];
	return(normalizedWeights);
    }


  /**
   * Initialize the Combinator.
   *
   * @param im The current indexing modalities.
   * @return A Combinator object.
   */
  
  static Combinator setCombinator(IndexingModalities im) {
    Combinator toBeReturned = null;
    Vector theModalities = im.getModalities();

    if (Support.DETAIL_TRACE <= Support.traceLevel)
      im.present();
    double[] normalizedWeights =
	requestNormalizedWeights(theModalities);

    String rep = "";
    do {
      if (Support.getPromptUser()) {
	Support.userPrompt("Linear (lin), Maximum (max) or Minimum (min): ");
	rep = Support.readLine().intern();
      } else {
	  rep = "lin"; // default
	rep.intern();
      }
    } while (rep != "lin" && rep != "max" && rep != "min");
    
    if (rep == "lin")
	toBeReturned =
	    new LinearCombinator(theModalities.size(), normalizedWeights);
    else
	toBeReturned =
	  new StructuredCombinator(rep, theModalities.size(), normalizedWeights);

    if (Support.DETAIL_TRACE<= Support.traceLevel){
	System.out.println("setCombinator:Combinator");
	toBeReturned.present(0, im);
	System.out.println("setCombinator:Combinator Brief");
	toBeReturned.presentBrief(8);
    }
    return (toBeReturned);
  }
    
  /**
   * Retrieve documents from a collection using the query supplied by the user
   * (invoked from main menu).
   */
  static void retrieve(RetrievalModalities rm, IndexingModalities im) {
    long timestamp = 0;
    long timestamp2 = 0;
    rm.setCombinatorStructure(setCombinator(im));
    ProxyDocument rd = null;
    int more = 1;
    while (more > 0) {
      char mode = '\0';
      if (Support.getPromptUser())
	do {
	  Support.userPrompt("Query mode (i=interactive, f=from file) : ");
	  try {
	    mode = (char)Support.readLine().charAt(0);
	  } catch (Exception error) {
	    mode = '\0';
	  }
	} while (mode == '\0');
      else
	mode = 'F';

      if (mode == 'i' || mode == 'I') {
	System.out.println("Query representative:"); 
	rd = dummy.getQueryDocument();
      } else if (mode == 'f' || mode == 'F') {
	  if (Support.DETAIL_TRACE <= Support.traceLevel)
	      Support.traceMessage("Query representative:\n"); 
	
	// Read fileName & read doc from file + create query document
	if (Support.getPromptUser()) {
	  queryName = "";
	  while (queryName == "") {
	    System.out.print("Doc title/filename : ");
	    try {
	      queryName = Support.readLine();
	      theConverter.setRawDocLocation(queryName);
	    } catch(Exception e) {
	      Support.errorMessage("This file doesn't exist. Error: " + e);
	      queryName = "";
	    }
	  }
	} else
	  try {
	    theConverter.setRawDocLocation(queryName);
	  } catch (FileNotFoundException fnfe) {
	    Support.errorMessage("defaults must be kidding...");
	  }
	try {
	  rd = theConverter.convert1();
	} catch ( IOException error ) {
	  Support.errorMessage("Error: " + error );
	}
	if (rd == null) {
	  Support.errorMessage("Sorry, can't read query");
	  return;
	}
      } else {
	Support.errorMessage("Sorry, illegal choice");
	return;
      }
      if (Support.DETAIL_TRACE <= Support.traceLevel)
	rd.present();
      
      System.out.println("Retrieving...");

      /*-----------------*
       * Do the retrieval
       *------------------*/
      timestamp = Support.getTimeInMillis(); 
      RetrievalResult result = rd.retrieve(rm, rd);
      timestamp2 = Support.getTimeInMillis();

      if (Support.COARSE_TRACE <= Support.traceLevel)
	  Support.traceMessage("Elapsed seconds to retrieve = " +
			       (timestamp2 - timestamp) / 1000);

      /*-----------------*
       * Sort the result
       *------------------*/
      timestamp = Support.getTimeInMillis(); 
      result.sort();
      timestamp2 = Support.getTimeInMillis(); 

      Vector rankedResultVector = result.getResultVector(); 

      int nbResults = result.getSize();
      if (Support.COARSE_TRACE <= Support.traceLevel)
	  Support.traceMessage("Elapsed seconds to sort "+ nbResults +
			       " documents = " +
			       (timestamp2-timestamp)/1000);

      System.out.println("Presenting result in brief format (" + nbResults + " documents)...");

      /*-----------------------------------*
       * Present the results in brief form
       *-----------------------------------*/
      DecimalFormat numberFormat = new DecimalFormat("0.000"); 
      for (int i = 1; i <= nbResults; i++) {
	ResultForDocMatchingQuery element = 
	  (ResultForDocMatchingQuery)rankedResultVector.elementAt(i - 1);
	System.out.print(i + ": (" + numberFormat.format(element.getRSV()) + ") ");
	((ProxyDocument)element.getDocument()).presentBrief();
      }
      
      more = 2;
      while (more > 0) {
	result.present(true); // TRUE:interactive mode
	Support.userPrompt("Give doc number to view (0 to quit) : ");
	
	if (Support.getPromptUser()) {
	  try {
	    more = Integer.parseInt(Support.readLine());
	  } catch (NumberFormatException error) {
	    Support.errorMessage("illegal input: " + error);
	  }
	} else
	  more--;
	
	if (more > 0) {
	  Document toPresent;
	  try {
	    toPresent = 
	      prepareDocPresent((ResultForDocMatchingQuery)rankedResultVector.elementAt(more - 1));
	  } catch (ArrayIndexOutOfBoundsException error) {
	    continue;
	  }
	  System.out.println("Presenting selected document ("+more+")...");
	  toPresent.present();
	  StringBuffer htmldoc = new StringBuffer();
	  toPresent.addHTML(htmldoc, 0, false);
	  try {
	    FileWriter output = new FileWriter("tmp.html");
	    output.write(new String(htmldoc));
	    output.close();
	  } catch (IOException error) {
	    Support.errorMessage("error: " + error );
	  }
	}
      }
      // This query finished, free up some space:
      if (Support.getPromptUser()) {
	Support.userPrompt("Another query ");
	if (confirmYN()) more = 1;
      }
    }
  }
   
  /**
   * Prompt user for yes or no response.
   * @return true if response from user is 'y', false otherwise.
   * Small helpfunction: could be moved to utils.[Ch]:
   */
  public static boolean confirmYN() {
    char answer = 'n';
    System.out.print("(y/n) ? ");
    answer = Support.readLine().charAt(0);
    return (answer == 'y');
  }
  
  /**
   * Print out data on memory usage
   */

  static void printMemUseStats(String message) {
    if (Support.MEDIUM_TRACE <= Support.traceLevel) {
      System.out.println("- - - - - - - - - - " + message);	
      Support.requestGarbageCollection();  
    }
  }
}
