package gov.nist.nlpir.irfapps.hci;

import gov.nist.nlpir.irf.document.Document;
import gov.nist.nlpir.irf.de.ProxyDeHtml;
import gov.nist.nlpir.irf.de.ProxyDePersonName;
import gov.nist.nlpir.irf.de.ProxyDeString;
import gov.nist.nlpir.irf.util.Support;

/**
 * An extension of Document for Bibliographic Documents.  
 * A test class using Human-Computer Interaction Paper Abstracts.
 *
 * @see gov.nist.nlpir.irf.document.Document
 *
 * @version $Revision: 1.5 $
 *
 * @serial title 
 * @serial author
 * @serial docAbstract
 *
 *
 *
 * @author
 * This software was produced by NIST, an agency of the
 * U.S. government, and by statute is not subject to copyright in the
 * United States.  Recipients of this software assume all
 * responsibilities associated with its operation, modification and
 * maintenance.
 */
public class HciDoc extends Document {


  /** serial version universal id - put here so Java does not insert one 
   * which may change due to revisions and make it impossible to deserialize
   * earlier versions of serialized objects */
  static final long serialVersionUID = 0;

  /** @serial */
  public ProxyDeString     title;

  /** @serial */
  public ProxyDePersonName author;

  /** @serial */
  public ProxyDeHtml       docAbstract; /* document abstract (Because 
                                              abstract" is a reserved word 
                                              we can't use "abstract" as an 
                                              instance variable name.) */
   
  /**
   * @param ttl         title of document
   * @param fitstName   Author First name
   * @param surname     Author Surname
   * @param abst        text of document abstract 
   */
  public HciDoc(String ttl, 
		String firstName, 
		String surname, 
		String abst) 
  {
    title = new ProxyDeString(ttl);
    author = new ProxyDePersonName(firstName,surname);
    docAbstract = new ProxyDeHtml(abst);
  }
    
    
  /**
   * @param ttl             title of document
   * @param name            author's name
   * @param abstr           document abstract
   */
  public HciDoc(ProxyDeString ttl, 
		ProxyDePersonName name,
		String abstr) 
  {
    title = ttl;
    author = name;
    docAbstract = new ProxyDeHtml(abstr);
  }
    
    
    /**
     * @param ttl             title of document
     * @param name            author's name
     * @param abstr           document abstract
     */
    public HciDoc(ProxyDeString ttl, 
		   ProxyDePersonName name,
		   ProxyDeHtml abstr) { 
      title = ttl;
      author = name;
      docAbstract = abstr;
    }

  public HciDoc() {
  }
    


  public String getTitle() {
    return title.toString();
  }
    

  /** Returns string representation of HCI Document with title,
   * author, and abstract.
   * @return The string representation of HCI Document.
   */
  public String toString() {
	return ("title: " + title + "\nauthor: " + author +
		"\nabstract: " + docAbstract);
  }

  public int hashCode() {
    return docAbstract.hashCode();
  }

  public boolean equals(Object o) {
    if (o instanceof HciDoc) {
      HciDoc temp = (HciDoc)o;
      return (this.title.equals(temp.title) &&
	      this.author.equals(temp.author) &&
	      this.docAbstract.equals(temp.docAbstract));
    }
    System.err.println("HciDoc.equals(): was expecting a HciDoc, received a " +
		       o.getClass().getName() + ": " + o);
    return false;
  }

  public Document getQueryDocument() {
    String title = "";
    String name1 = "";
    String name2 = "";
    String abstract_text = "";
    String tempString = "";

    // ------ Ask for title ---
    while (title.compareTo("") == 0) { 
      Support.userPrompt("Title or . to skip: ");
      title = Support.readLine(); 
    }
    if ((title.trim()).compareTo(".") == 0) 
      title = "";	      
    // ------ Ask for name1 ---
    while (name1.compareTo("") == 0) { 
      Support.userPrompt("First Name or . to skip: ");
      name1 = Support.readLine(); 
    }
    if ((name1.trim()).compareTo(".") == 0)
      name1 = "";	      
    // ------ Ask for name2 ---
    while (name2.compareTo("") == 0) { 
      Support.userPrompt("Second Name or . to skip: ");
      name2 = Support.readLine(); 
    }
    if ((name2.trim()).compareTo(".") == 0) 
      name2 = "";	   
    // ------ Ask for abstract_text ---
    while (abstract_text.compareTo("") == 0) { 
      Support.userPrompt("Abstract or . to skip: ");
      abstract_text = Support.readLine(); 
    }
    if ((abstract_text.trim()).compareTo(".") == 0) 
      abstract_text = "";	      

    return new HciDoc(title, name1, name2, abstract_text);
  }
  
  public void presentBrief() {
    System.out.println(title);
  }
}
