package gov.nist.nlpir.irfapps.hci;

import gov.nist.nlpir.irf.handle.Handle;
import gov.nist.nlpir.irf.proxy.Oid;
import gov.nist.nlpir.irf.proxy.VirtualProxy;
import gov.nist.nlpir.irfapps.hci.HciDocBrafHandle;
import gov.nist.nlpir.irf.broker.braf.DocumentBrafBroker;

import gov.nist.nlpir.irf.de.ProxyDeString;
import gov.nist.nlpir.irf.de.ProxyDePersonName;
import gov.nist.nlpir.irf.de.ProxyDeHtml;
import gov.nist.nlpir.irf.de.ProxyDeDouble;
import gov.nist.nlpir.irf.de.ProxyDeInteger;
import gov.nist.nlpir.irfapps.hci.HciDoc;
import gov.nist.nlpir.misc.io.BufferedRandomAccessFile;

/** 
 * This class is a file-based broker for HciDoc objects
 * objects
 * <br><br>
 * 
 * @version $Revision: 1.2 $ 	
 * 
 * @author
 * This software was produced by NIST, an agency of the U.S. government,  
 * and by statute is not subject to copyright in the United States.        
 * Recipients of this software assume all responsibilities associated      
 * with its operation, modification and maintenance.  
 */

public class HciDocBrafBroker extends DocumentBrafBroker {
  

  /** The static reference to singleton broker for this class of objects */
  private static HciDocBrafBroker broker = null;
  
  /** Working instance of a ProxyDE_... class */
  private static Class proxyDeHtml_Class = gov.nist.nlpir.irf.de.ProxyDeHtml.class;

  /** Working instance of a ProxyDE_... class */
  private static Class proxyDeStringClass = gov.nist.nlpir.irf.de.ProxyDeString.class;

  /** Working instance of a ProxyDE_... class */
  private static Class proxyDePersonNameClass = 
    gov.nist.nlpir.irf.de.ProxyDePersonName.class;



  /** 
   * Return the instance of the singleton class: HCI_doc file persistence 
   * broker, creating one first if one has not already been created.
   * @return the doc file persistence broker
   */      
  public static HciDocBrafBroker instance() {
    if (broker == null)
      broker = new HciDocBrafBroker();
    return broker;
  }
  

  
  /** 
   * Materializes the object associated with the designated argument.
   * This concrete version will handle any part of the materialization
   * which is specific to the object.
   * @param arg Information needed to materialize the object
   * @return Materialize the designated object
   */                 
  public Object currentRecordAsObject(Object arg) {
    HciDoc s = null;
    HciDocBrafHandle handle = (HciDocBrafHandle)arg;
    s = (HciDoc)readRecordFromFile(handle);
    return s;
  }
  


  /** 
   * Dematerializes the object associated with the designated argument.
   * This concrete version will handle any part of the materialization
   * which is specific to the object.
   * @param arg Object to be stored.
   * @return handle Information needed to materialize the object
   */                 
  public Handle currentObjectAsRecord(Object arg) {
    HciDocBrafHandle handle = new HciDocBrafHandle();
    writeRecordToFile(handle, arg);
    return handle;
  }



  /**
   * Overrides the default writeObject with tailored code
   * @parm obj object to write
   * @parm braf buffered random access file to write the object to
   */
  public void writeObject(Object obj, BufferedRandomAccessFile braf) throws 
  java.io.IOException {
    HciDoc beingStored = (HciDoc)obj;
    beingStored.title.getObjectIdentifier().writeTo(braf);
    beingStored.author.getObjectIdentifier().writeTo(braf);
    beingStored.docAbstract.getObjectIdentifier().writeTo(braf);
  }



  /**
   * Overrides the default readObject with tailored code
   * @parm braf buffered random access file to write the object to
   * @return the deserialized object
   */
  public Object readObject(BufferedRandomAccessFile braf) throws 
  java.io.IOException {
    ProxyDeString ttl =
      (ProxyDeString)VirtualProxy.
      getProxyFor(proxyDeStringClass, Oid.readFrom(braf));
    ProxyDePersonName name =
      (ProxyDePersonName)VirtualProxy.
      getProxyFor(proxyDePersonNameClass, Oid.readFrom(braf));
    ProxyDeHtml abstr =
      (ProxyDeHtml)VirtualProxy.
      getProxyFor(proxyDeHtml_Class, Oid.readFrom(braf));

    HciDoc result = new HciDoc(ttl, name, abstr);
    result.setProxyRefsCountedTrue();
    return result;
  }
}
