package gov.nist.nlpir.irfapps.hci;

import gov.nist.nlpir.irfapps.hci.HciDoc;
import gov.nist.nlpir.irf.document.ProxyDocument;
import gov.nist.nlpir.irf.de.ProxyDePersonName;
import gov.nist.nlpir.irf.de.ProxyDeString;
import gov.nist.nlpir.irf.broker.PersistenceBroker;
import gov.nist.nlpir.irfapps.hci.HciDocBrafBroker;

import java.io.PrintStream;

/**
 * An stand-in for an HciDoc
 *
 * @see gov.nist.nlpir.irfapps.hci.HciDoc
 *
 * @version $Revision: 1.5 $
 *
 * @author
 * This software was produced by NIST, an agency of the
 * U.S. government, and by statute is not subject to copyright in the
 * United States.  Recipients of this software assume all
 * responsibilities associated with its operation, modification and
 * maintenance.
 */
public class ProxyHciDoc extends ProxyDocument {

  // Constructor for deserialization
  public ProxyHciDoc() {
    setRealObject(null);
  }


  public ProxyHciDoc(HciDoc realDoc) {
    setRealObject(realDoc);
  }


  /**
   * @param ttl         title of document
   * @param firstName   Author First name
   * @param surname     Author Surname
   * @param abst        text of document abstract 
   */
  public ProxyHciDoc(String ttl, 
		     String firstName, 
		     String surname, 
		     String abst) { 
    setRealObject(new HciDoc(ttl,firstName,surname,abst));
  }

   

  /**
   * @param ttl             title of document
   * @param name            autho's name
   * @param abstr           document abstract
   */
  public ProxyHciDoc(ProxyDeString ttl, 
		     ProxyDePersonName name,
		     String  abstr) { 
    setRealObject(new HciDoc(ttl, name, abstr));
  }
    

  public String getTitle() {
    return ((HciDoc)getRealObject()).getTitle();
  }

    
  /** Returns string representation of HCI Document with title,
   * author, and abstract.
   * @return The string representation of HCI Document.
   */
  public String toString() {
    return ((HciDoc)getRealObject()).toString();	
  }


  /** 
   *Creates a  broker for this proxy object and remember it
   * @return the broker for this object
   */          
  public PersistenceBroker createBroker(){
    setBroker(HciDocBrafBroker.instance());
    return(getBroker());
  }



  /** 
   *Creates a document for use as a query
   * @return the document
   */          
  public ProxyDocument getQueryDocument() {
    return new ProxyHciDoc((HciDoc)(new HciDoc()).getQueryDocument());
  }


  public void presentBrief() {
    System.out.println(((HciDoc)getRealObject()).getTitle());
  }
}
