package gov.nist.nlpir.irfapps.trec;

import gov.nist.nlpir.irf.document.Document;
import gov.nist.nlpir.irf.de.ProxyDeString;

/**
 * An extension of Document that should be common to all TREC documents.
 * Every TREC Collection may extend this document to have a particular type
 * containing the fields needed.
 *
 * @see gov.nist.nlpir.irf.document.Document
 *
 * @version $Revision: 1.3 $
 *
 * @serial docNumber
 *
 *
 *
 * @author
 * This software was produced by NIST, an agency of the
 * U.S. government, and by statute is not subject to copyright in the
 * United States.  Recipients of this software assume all
 * responsibilities associated with its operation, modification and
 * maintenance.
 */

abstract public class TrecDocument extends Document {


  /** serial version universal id - put here so Java does not insert one 
   * which may change due to revisions and make it impossible to deserialize
   * earlier versions of serialized objects */
  static final long serialVersionUID = 0;

  /** @serial */
  public ProxyDeString docNumber;
  

  public TrecDocument() {
    docNumber = null;
  }

  /**
   * @param docNr document number
   */
  public TrecDocument(String docNr) {
    docNumber = new ProxyDeString(docNr);
  }
  
  /**
   * @param docNr document number
   */
  public TrecDocument(ProxyDeString docNr) {
    docNumber = docNr;
  }
  
  /**
   * Returns string representation of TREC Document
   * <code>#: docNumber</code>
   */
  public String toString() {
    return ("#: " + docNumber);
  }
  
  public int hashCode() {
    return docNumber.hashCode();
  }
  
  public boolean equals(Object o) {
    if (o instanceof TrecDocument) {
      TrecDocument temp = (TrecDocument)o;
      return (this.docNumber.equals(temp.docNumber));
    }
    System.err.println("TrecDocument.equals(): was expecting a TrecDocument, received a " +
		       o.getClass().getName() + ": " + o);
    return false;
  }

  /* commented out as long as TrecDocument is abstract.

  public Document getQueryDocument() {
    String docNr = "";
      System.out.print("Type . to leave a field empty (no value)\n" +
		       "Doc Number       : "); 
      docNr = Support.readLine();
    return new TrecDocument(docNr);
  }
  */
}
