package gov.nist.nlpir.irfapps.trec.cran;

import gov.nist.nlpir.irf.handle.Handle;
import gov.nist.nlpir.irfapps.trec.cran.CranfieldDocumentBrafHandle;
import gov.nist.nlpir.irf.broker.braf.DocumentBrafBroker;

import gov.nist.nlpir.irfapps.trec.cran.CranfieldDocument;
import gov.nist.nlpir.misc.io.BufferedRandomAccessFile;

/** 
 * This class is a flat file persistent storage broker for CranfieldDocument
 *  objects
 * <br><br>
 * 
 * @version $Revision: 1.1 $ 	
 * 
 * @author
 * This software was produced by NIST, an agency of the U.S. government,  
 * and by statute is not subject to copyright in the United States.        
 * Recipients of this software assume all responsibilities associated      
 * with its operation, modification and maintenance.  
 */

public class CranfieldDocumentBrafBroker extends DocumentBrafBroker {
  
  private static CranfieldDocumentBrafBroker broker = null;
  
  /** 
   * Return the instance of the singleton class: doc file persistence broker, 
   * creating one first if one has not already been created.
   * @return the doc file persistence broker
   */      
  
  public static CranfieldDocumentBrafBroker instance() {
    if (broker == null)
      broker = new CranfieldDocumentBrafBroker();
    return broker;
  }
  
  
  /** 
   * Materialize the object associated with the designated argument.
   * This concrete version will handle any part of the materialization
   * which is specific to the object.
   * @param arg Information needed to materialize the object
   * @return Materialize the designated object
   */                 
  
  public Object currentRecordAsObject(Object arg) {
    CranfieldDocument s = null;
    CranfieldDocumentBrafHandle handle = (CranfieldDocumentBrafHandle)arg;
    s = (CranfieldDocument)readRecordFromFile(handle);
    return s;
  }
  
  /** 
   * Dematerialize the object associated with the designated argument.
   * This concrete version will handle any part of the materialization
   * which is specific to the object.
   * @param arg Object to be stored.
   * @return handle Information needed to materialize the object
   */                 
  
  public Handle currentObjectAsRecord(Object arg) {
    CranfieldDocumentBrafHandle handle = new CranfieldDocumentBrafHandle();
    writeRecordToFile(handle, arg);
    return handle;
  }

}
