package gov.nist.nlpir.irfapps.trec.fbis;

import gov.nist.nlpir.irfapps.trec.TrecDocument;

import gov.nist.nlpir.irf.document.Document;
import gov.nist.nlpir.irf.de.ProxyDeString;
import gov.nist.nlpir.irf.de.ProxyDeHtml;
import gov.nist.nlpir.irf.util.Support;

/**
 * A specialization of the TREC document to model a document from the TREC
 * Foreign Broadcast Information Service (FBIS) collection.
 *
 * @see gov.nist.nlpir.irf.document.Document
 *
 * @version $Revision: 1.4 $
 *
 * @serial title
 * @serial docAbstract
 * @serial text
 *
 *
 *
 * @author
 * This software was produced by NIST, an agency of the
 * U.S. government, and by statute is not subject to copyright in the
 * United States.  Recipients of this software assume all
 * responsibilities associated with its operation, modification and
 * maintenance.
 */

public class FbisDocument extends TrecDocument {
  

  /** serial version universal id - put here so Java does not insert one 
   * which may change due to revisions and make it impossible to deserialize
   * earlier versions of serialized objects */
  static final long serialVersionUID = 0;

  /** @serial */
  public ProxyDeString title;

  /** @serial */
  public ProxyDeString docAbstract;

  /** @serial */
  public ProxyDeHtml text;
  

  public FbisDocument() {
    super();
    title = null;
    docAbstract = null;
    text = null;
  }

  public FbisDocument(String docNr, String titl, 
		       String abstr, String txt) {
    super(docNr);
    title  = new ProxyDeString(titl);
    docAbstract  = new ProxyDeString(abstr);
    text  = new ProxyDeHtml(txt);
  }

  public FbisDocument(ProxyDeString docNr, String titl, 
		       String abstr, String txt) {
    super(docNr);
    title  = new ProxyDeString(titl);
    docAbstract  = new ProxyDeString(abstr);
    text  = new ProxyDeHtml(txt);
  }

  public FbisDocument(ProxyDeString docNr, ProxyDeString titl, 
		       ProxyDeString abstr, ProxyDeHtml txt) {
    super(docNr);
    title  = titl;
    docAbstract  = abstr;
    text  = txt;
  }
  
  
  /**
   * Returns string representation of FBIS Document
   */
  public String toString() {
    return ("\nD o c n o :       " + docNumber +
	    "\nT i t l e :       " + title +
	    "\nA b s t r a c t : " + docAbstract +
	    "\nT e x t :         " + text);
  }
  

  public boolean equals(Object o) {
    if (o instanceof FbisDocument)
      return (this.docNumber.equals(((FbisDocument)o).docNumber));

    System.err.println("FbisDocument.equals(): was expecting an FbisDocument, received a " +
		       o.getClass().getName() + ": " + o);
    return false;
  }

  public Document getQueryDocument() {
    String docNr = "";
    String title = "";
    String abstr = "";
    String txt = "";
    System.out.print("Type . to leave a field empty (no value)\n" +
		     "Doc Number       : "); 
    docNr = Support.readLine();
    System.out.print("Title  : "); 
    title = Support.readLine(); 
    System.out.print("Abstract : "); 
    abstr = Support.readLine();
    System.out.print("Text    : ");
    txt = Support.readLine();
    return new FbisDocument(docNr, title, abstr, txt);
  }

  public void presentBrief() {
    if (docNumber != null) 
      {
      String docnum = docNumber.getValue();
      String prnstr = docnum.substring(0, Math.min(docnum.length(), 20));
      System.out.print(prnstr);
      }
    if (title != null) {
      String ttl = title.getValue();
      String prnTtl = ttl.substring(0, Math.min(ttl.length(), 60));
      System.out.println(prnTtl);
    } else if (docAbstract != null) {
      String abstr = docAbstract.getValue();
      String prnAbstr = abstr.substring(0, Math.min(abstr.length(), 50));
      System.out.println("Doc with no title (abstract: " + prnAbstr + "...)");
    } else if (text != null) {
      String text = this.text.getValue();
      String prnText = text.substring(0, Math.min(text.length(), 50));
      System.out.println("Doc with no title (text: " + prnText + "...)");
    } else
      System.out.println("Doc with no title, abstract or text.");
      
  }
}
