package gov.nist.nlpir.irfapps.trec.fbis;

import gov.nist.nlpir.irf.proxy.Oid;
import gov.nist.nlpir.irf.handle.Handle;
import gov.nist.nlpir.irf.proxy.VirtualProxy;
import gov.nist.nlpir.irf.broker.braf.DocumentBrafBroker;
import gov.nist.nlpir.irfapps.trec.fbis.FbisDocumentBrafHandle;

import gov.nist.nlpir.irf.de.ProxyDeString;
import gov.nist.nlpir.irf.de.ProxyDeHtml;
import gov.nist.nlpir.irfapps.trec.fbis.FbisDocument;
import gov.nist.nlpir.misc.io.BufferedRandomAccessFile;

/** 
 * This class is a file-based broker for FbisDocument objects
 * <br><br>
 * 
 * @version $Revision: 1.1 $ 	
 * 
 * @author
 * This software was produced by NIST, an agency of the U.S. government,  
 * and by statute is not subject to copyright in the United States.        
 * Recipients of this software assume all responsibilities associated      
 * with its operation, modification and maintenance.  
 */

public class FbisDocumentBrafBroker extends DocumentBrafBroker {
  

  /** The static reference to singleton broker for this class of objects */
  private static FbisDocumentBrafBroker broker = null;
  
  /** Working instance of ProxyDE_... */
  private static Class proxyDeHtml_Class = gov.nist.nlpir.irf.de.ProxyDeHtml.class;

  /** Working instance of ProxyDE_... */
  private static Class proxyDeStringClass = gov.nist.nlpir.irf.de.ProxyDeString.class;



  /** 
   * Return the instance of the singleton class: FBIS doc file persistence 
   * broker, creating one first if one has not already been created.
   * @return the doc file persistence broker
   */      
  public static FbisDocumentBrafBroker instance() {
    if (broker == null)
      broker = new FbisDocumentBrafBroker();
    return broker;
  }
  

  
  /** 
   * Materializes the object associated with the designated argument.
   * This concrete version will handle any part of the materialization
   * which is specific to the object.
   * @param arg Information needed to materialize the object
   * @return Materialize the designated object
   */                 
  public Object currentRecordAsObject(Object arg) {
    FbisDocument s = null;
    FbisDocumentBrafHandle handle = (FbisDocumentBrafHandle)arg;
    s = (FbisDocument)readRecordFromFile(handle);
    return s;
  }
  


  /** 
   * Dematerializes the object associated with the designated argument.
   * This concrete version will handle any part of the materialization
   * which is specific to the object.
   * @param arg Object to be stored.
   * @return handle Information needed to materialize the object
   */                 
  public Handle currentObjectAsRecord(Object arg) {
    FbisDocumentBrafHandle handle = new FbisDocumentBrafHandle();
    writeRecordToFile(handle, arg);
    return handle;
  }



  /**
   * Overrides the default writeObject with tailored code
   * @parm obj object to write
   * @parm braf buffered random access file to write the object to
   */
  public void writeObject(Object obj, BufferedRandomAccessFile braf) throws 
  java.io.IOException {
    FbisDocument beingStored = (FbisDocument)obj;

    if (beingStored.docNumber != null)
      beingStored.docNumber.getObjectIdentifier().writeTo(braf);
    else
      Oid.writeNullOid(braf);

    if (beingStored.title != null)
      beingStored.title.getObjectIdentifier().writeTo(braf);
    else
      Oid.writeNullOid(braf);

    if (beingStored.docAbstract != null)
      beingStored.docAbstract.getObjectIdentifier().writeTo(braf);
    else
      Oid.writeNullOid(braf);

    if (beingStored.text != null)
      beingStored.text.getObjectIdentifier().writeTo(braf);
    else
      Oid.writeNullOid(braf);
  }


  /**
   * Overrides the default readObject with tailored code
   * @parm braf buffered random access file to write the object to
   * @return the deserialized object
   */
  public Object readObject(BufferedRandomAccessFile braf) throws 
  java.io.IOException {
    ProxyDeString docNr       = null;
    ProxyDeString title       = null;
    ProxyDeString docAbstract = null;
    ProxyDeHtml   text        = null;

    Oid lastReadOid = Oid.readFrom(braf);
    if (lastReadOid != null)
      docNr = (ProxyDeString)VirtualProxy.
	getProxyFor(proxyDeStringClass, lastReadOid);

    lastReadOid = Oid.readFrom(braf);
    if (lastReadOid != null)
      title = (ProxyDeString)VirtualProxy.
	getProxyFor(proxyDeStringClass, lastReadOid);

    lastReadOid = Oid.readFrom(braf);
    if (lastReadOid != null)
      docAbstract = (ProxyDeString)VirtualProxy.
	getProxyFor(proxyDeStringClass, lastReadOid);

    lastReadOid = Oid.readFrom(braf);
    if (lastReadOid != null)
      text = (ProxyDeHtml)VirtualProxy.
	getProxyFor(proxyDeHtml_Class, lastReadOid);

    FbisDocument result = new FbisDocument(docNr, title, docAbstract, text);
    result.setProxyRefsCountedTrue();
    return result;
  }
}
